// ###########################################################################
// File: EArrayIteratorT.h
// (c)Copyright 03/1999, Gaia Creation, Baptiste Lepilleur
// ###########################################################################

#if !defined(AFX_EARRAYITERATOR_H__CDD75B81_DF76_11D2_BD6D_0040053AD648__INCLUDED_)
#define AFX_EARRAYITERATOR_H__CDD75B81_DF76_11D2_BD6D_0040053AD648__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

/* History:
 * 21/03/1999: - Creation.
 * 30/03/1999: - attach() not use a const array. !!! casted to non-const !!!
 *               This is necessary to allow attaching to a const object.
 *
 */


#include "ERandomItT.h"
#include "EArray.h"

template<class TElm, class EAllocator>
class EArrayIteratorT : public ERandomItT<TElm>
{
public:
  /*! Constructs an iterator.
   *
   *  The iterator must be binded to an array using attach().
   */
	EArrayIteratorT();

  /*! Constructs an iterator and binds it to an array.
   * @param array Array the iterator iterates on.
   */
  EArrayIteratorT( const EArrayT<TElm,EAllocator> &array );

  /*! Destructor.
   */
  virtual ~EArrayIteratorT() {}

  /*! Attaches the iterator to an array.
   * @param array Array the iterator iterates on.
   */
  void attach( const EArrayT<TElm,EAllocator> &array );

  /*! Sets the iterator at the beginning of the table.
   */
  void begin();

  /*! Tests if there is still some element to be read.
   * @return \c false if there is no more element to read, otherwise returns \c true.
   */
  bool isEnd() const;

  /*! Moves the iterator to the next association.
   */
  void next();

  /*! Gets the item at the current position of the iterator.
   * @return Item at the current position of the iterator.
   */
  TElm& item();

  /*! Gets the item at the current position of the iterator.
   * @return Item at the current position of the iterator.
   */
  const TElm& item() const;

  /*! Gets the current position of the iterator in the array.
   * @return Current position of the iterator in the array. -1 is returned if the
   *         Iterator is not binded.
   */
  SL pos() const;

  /*! Gets the number of items.
   * @return Number of items.
   */
  SL count() const;

  /*! Set the iterator at a given position.
   * @param nPos New position of the iterator (zero based).
   */
  void seekTo( SL nPos );

  /*! Get an element at a given position.
   * @param nPos Position of the element.
   */
  TElm &at( SL nPos );

private:
  /**@shapeType DependencyLink*/
  /*#  EArrayT lnkUnnamed*/
  EArrayT<TElm,EAllocator> *_pArray;
  SL _nCurPos;
};


// Implementation
// ###########################################################################

template<class TElm, class EAllocator>
EArrayIteratorT<TElm,EAllocator>::EArrayIteratorT() :
    _pArray( NULL ),
    _nCurPos( -1 )
{
}


template<class TElm, class EAllocator>
EArrayIteratorT<TElm,EAllocator>::EArrayIteratorT( const EArrayT<TElm,EAllocator> &array ) :
    _pArray( &(EArrayT<TElm,EAllocator> &)array )
{
  begin();
}


template<class TElm, class EAllocator>
void 
EArrayIteratorT<TElm,EAllocator>::attach( const EArrayT<TElm,EAllocator> &array )
{
  _pArray = &(EArrayT<TElm,EAllocator> &)array;
  begin();
}


template<class TElm, class EAllocator>
inline void 
EArrayIteratorT<TElm,EAllocator>::begin()
{
  EASSERT( _pArray );     // No array attached !
  _nCurPos =0;
}


template<class TElm, class EAllocator>
inline bool 
EArrayIteratorT<TElm,EAllocator>::isEnd() const
{
  EASSERT( _pArray );     // No array attached !
  return _pArray->count() <= _nCurPos;
}


template<class TElm, class EAllocator>
inline void 
EArrayIteratorT<TElm,EAllocator>::next()
{
  EASSERT( _pArray );     // No array attached !
  _nCurPos++;
}


template<class TElm, class EAllocator>
inline TElm& 
EArrayIteratorT<TElm,EAllocator>::item()
{
  EASSERT( _pArray );     // No array attached !
  return _pArray->at( _nCurPos );
}


template<class TElm, class EAllocator>
inline const TElm& 
EArrayIteratorT<TElm,EAllocator>::item() const
{
  EASSERT( _pArray );     // No array attached !
  return _pArray->at( _nCurPos );
}


template<class TElm, class EAllocator>
inline SL 
EArrayIteratorT<TElm,EAllocator>::count() const
{
  EASSERT( _pArray );     // No array attached !
  return _pArray->count();
}


template<class TElm, class EAllocator>
inline void 
EArrayIteratorT<TElm,EAllocator>::seekTo( SL nPos )
{
  EASSERT( nPos >= 0  &&  nPos <= count() );
  _nCurPos = nPos;
}


template<class TElm, class EAllocator>
inline TElm &
EArrayIteratorT<TElm,EAllocator>::at( SL nPos )
{
  EASSERT( _pArray );
  _nCurPos = nPos;
  return _pArray->at( nPos );
}


template<class TElm, class EAllocator>
inline SL 
EArrayIteratorT<TElm,EAllocator>::pos() const
{
  return _nCurPos;
}

#endif // !defined(AFX_EARRAYITERATOR_H__CDD75B81_DF76_11D2_BD6D_0040053AD648__INCLUDED_)
