// EBaseRef.h: interface for the EBaseRef class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EBASEREF_H__A1E5DBE1_480B_11D2_AE43_F8FAF6E5B561__INCLUDED_)
#define AFX_EBASEREF_H__A1E5DBE1_480B_11D2_AE43_F8FAF6E5B561__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "ERefObj.h"

/*! \brief This class provides a mean to acces objects using reference counting.
 */
class EBaseRef
{
public:
	/*! Constructs a \c NULL reference.
   */
  EBaseRef();
  
  /*! Constructs a reference from a pointer to an object.
   * @param p Object that is referenced.
   */
	EBaseRef( ERefObj *p );

  /*! Constructs a reference from another reference.
   * @param ref Reference that is copied.
   */
  EBaseRef( const EBaseRef& ref );

  /*! Release the reference to the object.
   */
	virtual ~EBaseRef();

  /*! Release the reference to the object.
   *
   * After the release, the reference is a \c NULL reference.
   */
	void release();
#if 0    
	  ERefObj& operator *() const;

	  ERefObj** operator &() const;
#endif
	ERefObj* operator->() const;

	EBaseRef &operator=( ERefObj *p );  

  EBaseRef &operator=( const EBaseRef& ref );

  operator bool() const;

  bool isNull() const;

protected:
  /*! Pointer on the object that is referenced.
   */
  ERefObj *_p;
};




inline 
EBaseRef::EBaseRef() :
    _p( NULL )
{
}


inline 
EBaseRef::EBaseRef( ERefObj *p ) :
    _p( p )
{
  if ( p )
    p->incRef();
}


inline 
EBaseRef::EBaseRef( const EBaseRef& ref ) :
    _p( ref._p )
{
  if ( _p )
    _p->incRef();
}


inline 
EBaseRef::~EBaseRef()
{
  if ( _p )
    _p->decRef();
}


inline void
EBaseRef::release()
{
  if ( _p )
  {
    _p->decRef();
    _p = NULL;
  }
}


#if 0
  inline ERefObj& 
  EBaseRef::operator*()
  {
    EASSERT( _p ); 
    return *_p; 
  }


  inline ERefObj** 
  EBaseRef::operator &() 
  { 
    EASSERT( _p ); 
    return &_p; 
  }
#endif


inline ERefObj *
EBaseRef::operator ->() const
{ 
  EASSERT( _p);
  return _p; 
}


inline
EBaseRef::operator bool () const
{
  return  _p != NULL;
}


inline  EBaseRef &
EBaseRef::operator =( ERefObj *p )
{
  if ( _p )
    _p->decRef();
  _p = p;
  if ( p )
    p->incRef();
  return *this;
}


inline EBaseRef &
EBaseRef::operator=( const EBaseRef& ref )
{
  if ( _p )
    _p->decRef();
  _p = ref._p;
  if ( _p )
    _p->incRef();
  return *this;
}


inline bool 
EBaseRef::isNull() const
{
  return _p == NULL;
}

#endif // !defined(AFX_EBASEREF_H__A1E5DBE1_480B_11D2_AE43_F8FAF6E5B561__INCLUDED_)
