// EHashBiBaseItT.h: interface for the EHashBiBaseItT class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EHASHBIBASEITT_H__ECDCC8F2_7768_11D3_BE9F_0040053AD648__INCLUDED_)
#define AFX_EHASHBIBASEITT_H__ECDCC8F2_7768_11D3_BE9F_0040053AD648__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000


#include "EIteratorT.h"

template<class Key, class Item, class KeyArg, class ItemArg>
class EHashBiItT : public EIteratorT<Item>
{
public:
  typedef EHashBiBaseT<Key,Item,KeyArg,ItemArg> ThisHashBase;

  EHashBiItT( const EHashBiBaseT<Key,Item,KeyArg,ItemArg> &hashTable );
  EHashBiItT();

  void attach(  const EHashBiBaseT<Key,Item,KeyArg,ItemArg> &hashTable );

  /** Set the iterator at the beginning of the table.
   * If there is no element in the table then isEnd() will return true.
   */
  void begin();

  /** Test if the iterator is beyond the last association.
   */
  bool isEnd() const;

  /** Remove the association at the current position of the iterator.
   */
  void remove();

  /** Move the iterator to the next association.
   */
  void next();

  /** Get the item at the current position of the iterator.
   * @return item at the current position of the iterator.
   */
  Item& item();

  /** Get the item at the current position of the iterator.
   * @return item at the current position of the iterator.
   */
  const Item& item() const;

  /** Get the key at the current position of the iterator.
   * @return key at the current position of the iterator.
   */
  const Key& key() const;

  /*! Get the numbers of items.
   * @return Number of items.
   */
  SL count() const;

private:
  typedef ThisHashBase::ELcAsso ThisLcAsso;

  /** Search the next index with an association.
   * The search begins at the index _nIndex +1.
   */
  void searchNextIndex();

  /// Hash table the iterator operate on.
  const EHashBiBaseT<Key,Item,KeyArg,ItemArg> *_pHashTable;

  /// Current index in association table.
  SL _nIndex;

  /// Current Association for the list at nIndex.
  const EHashBiBaseT<Key,Item,KeyArg,ItemArg>::ELCAsso *_pAsso;
};



// Template implementation:
// #####################Last change: 28/03/98 20h01mn#########################

template<class Key, class Item, class KeyArg, class ItemArg>
EHashBiItT<Key,Item,KeyArg,ItemArg>::EHashBiItT(
    const EHashBiBaseT<Key,Item,KeyArg,ItemArg> &hashTable )
{
  _pHashTable = &hashTable;
  _nIndex = 0;
  _pAsso = ENIL;
  begin();
}


template<class Key, class Item, class KeyArg, class ItemArg>
EHashBiItT<Key,Item,KeyArg,ItemArg>::EHashBiItT()
{
  _pHashTable = NULL;
  _nIndex = 0;
  _pAsso = ENIL;
}


template<class Key, class Item, class KeyArg, class ItemArg>
void
EHashBiItT<Key,Item,KeyArg,ItemArg>::attach(
    const EHashBiBaseT<Key,Item,KeyArg,ItemArg> &hashTable )
{
  EASSERT( pHashTable );
  _pHashTable = pHashTable;
  _nIndex = 0;
  _pAsso = ENIL;
  begin();
}

template<class Key, class Item, class KeyArg, class ItemArg>
void EHashBiItT<Key,Item,KeyArg,ItemArg>::begin()
{
  EASSERT( _pHashTable );
  _nIndex = -1;
  searchNextIndex();
}

template<class Key, class Item, class KeyArg, class ItemArg>
bool EHashBiItT<Key,Item,KeyArg,ItemArg>::isEnd() const
{
  EASSERT( _pHashTable );
  return _pAsso == ENIL;
}


template<class Key, class Item, class KeyArg, class ItemArg>
void EHashBiItT<Key,Item,KeyArg,ItemArg>::remove()
{
  EASSERT( _pHashTable );
  if ( isEnd() )
    return;

  const ThisLcAsso *pAsso = _pHashTable->_aAssoByKey[ _nIndex ];
  const ThisLcAsso **ppPrevAsso = &_pHashTable->_aAssoByKey[ nIndexKey ];
  while ( pAsso != _pAsso )                 // Find the pointed association
  {
    ppPrevAsso = &pAsso->_pNextByKey;
    pAsso = pAsso->_pNextByKey;
    EASSERT( pAsso );
  }

  _pAsso = *ppPrevAsso;     // Skip to next association

  // Delete entry for look up by item.
  ELcAsso pAssoThis = pAsso;      
  SL nIndexItem = _pHashTable->doGetKeyItem( pAsso->_item ) % _aAssoByItem.count();
  ThisLcAsso *pAsso = _pHashTable->_aAssoByItem[ nIndexItem ];
  ppPrevAsso = &_aAssoByItem[ nIndexItem ];
  while ( pAsso != pAssoThis )
  {
    ppPrevAsso = &pAsso->_pNextByItem;
    pAsso = pAsso->_pNextByItem;
    EASSERT( pAsso );     // inconsistency between key and item look up table.
  }

  *ppPrevAsso = pAsso->_pNextByItem;
  _pHashTable->doFreeAsso( pAssoThis->_key, pAssoThis->_item );
  delete pAssoThis;
  --_nNbAsso;

  if ( !_pAsso )                  // if no more in the list for this index
  {                               // then search next.
    searchNextIndex();
  }
}


template<class Key, class Item, class KeyArg, class ItemArg>
void EHashBiItT<Key,Item,KeyArg,ItemArg>::next()
{
  EASSERT( _pHashTable );
  if ( _pAsso )
  {
    _pAsso = _pAsso->_pNext;
    if ( !_pAsso )          // end of current index reached ?
      searchNextIndex();
  }
}


template<class Key, class Item, class KeyArg, class ItemArg>
void EHashBiItT<Key,Item,KeyArg,ItemArg>::searchNextIndex()
{
  EASSERT( _pHashTable );
  SL nSize = _pHashTable->size();
  _nIndex++;
  for ( ;_nIndex < nSize; _nIndex++ )
  {
    if ( _pHashTable->_aAsso[ _nIndex ] )
    {
      _pAsso = _pHashTable->_aAsso[ _nIndex ];
      return;
    }
  }
  _pAsso = ENIL;
}

template<class Key, class Item, class KeyArg, class ItemArg>
const Key& EHashBiItT<Key,Item,KeyArg,ItemArg>::key() const
{
  EASSERT( _pHashTable );
  EASSERT( _pAsso );
  return _pAsso->_key;
}

template<class Key, class Item, class KeyArg, class ItemArg>
const Item& EHashBiItT<Key,Item,KeyArg,ItemArg>::item() const
{
  EASSERT( _pHashTable );
  EASSERT( _pAsso );
  return _pAsso->_item;
}

template<class Key, class Item, class KeyArg, class ItemArg>
Item& EHashBiItT<Key,Item,KeyArg,ItemArg>::item()
{
  EASSERT( _pHashTable );
  EASSERT( _pAsso );
  return *(Item *)&_pAsso->_item;
}


template<class Key, class Item, class KeyArg, class ItemArg>
inline SL 
EHashBiItT<Key,Item,KeyArg,ItemArg>::count() const
{
  EASSERT( _pHashTable );
  return _pHashTable->count();
}


#endif // !defined(AFX_EHASHBIBASEITT_H__ECDCC8F2_7768_11D3_BE9F_0040053AD648__INCLUDED_)
