// EHashBiBaseT.h: interface for the EHashBiBaseT class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EHASHBIBASET_H__BF9A2C73_750D_11D3_BE99_0040053AD648__INCLUDED_)
#define AFX_EHASHBIBASET_H__BF9A2C73_750D_11D3_BE99_0040053AD648__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "EArray.hpp"
#include "EDynArray.h"

/*
  check get makeing (call to doGetKeyItem...)
 */


template<class Key, class Item, class KeyArg, class ItemArg>
class EBaseHashIteratorT;

template<class Key, class Item, class KeyArg, class ItemArg>
class EHashBiBaseT
{
#ifdef _SUPPORT_TEMPLATE_FRIEND
  friend class EBaseHashIteratorT<Key,Item,KeyArg,ItemArg>;
#endif

public:
  typedef EBaseHashIteratorT<Key,Item,KeyArg,ItemArg> EIterator;
  typedef Key ThisKey;
  typedef Item ThisItem;
  typedef KeyArg ThisKeyArg;
  typedef ItemArg ThisItemArg;
  typedef EHashBiBaseT<Key,Item,KeyArg,ItemArg> ThisHashBase;

  /*! Constructor.
   * @param nSize initial size of the table.
   */
  EHashBiBaseT( SL nSize = 3 );

  /*! Destructor
   */
  virtual ~EHashBiBaseT();

  /*! Lookup a item using its key.
   * @param key key associated to the item.
   * @return reference on item.
   */
  Item& operator[]( KeyArg key );

  /* Lookup a item using its key.
   * @param key key associated to the item.
   * @return reference on item.
   */
//  const Item& operator[]( const KeyArg key ) const;

  /*! Lookup a item using its key.
   * @param key key associated to the item.
   * @return pointer on item if found. NULL if no item associated to key.
   */
  Item *lookUp( KeyArg key );

  /*! Lookup a item using its key.
   * @param key key associated to the item.
   * @return pointer on item if found. NULL if no item associated to key.
   */
  const Item *lookUp( KeyArg key ) const;

  /*! Lookup a key using its item.
   * @param item item associated to the item.
   * @return Pointer on the key if found. \c NULL if no item associated to key.
   */
  Key *lookUpByItem( ItemArg item );

  /*! Lookup a key using its item.
   * @param item item associated to the item.
   * @return Pointer on the key if found. \c NULL if no item associated to key.
   */
  const Key *lookUpByItem( ItemArg item ) const;

  /*! Lookup a item key using its key.
   * @param key key associated to the item.
   * @return pointeur on key if found. NULL if no item associated to key.
   */
  const Key *lookUpKey( KeyArg key ) const;

  /*! Lookup a item key using its key.
   * @param key key associated to the item.
   * @return pointeur on key if found. NULL if no item associated to key.
   */
  const Item *lookUpItem( ItemArg item ) const;

  /*! Add an item to the association table.
   * @param key associated to the item.
   * @param item item to add.
   */
  void add( KeyArg key, ItemArg item );

  /*! Remove an item from the association table.
   * @param key key associated to the item to remove.
   */
  void remove( KeyArg key );

  /*! Remove an item from the association table.
   * @param key key associated to the item to remove.
   */
  void removeByItem( ItemArg item );

  /*! Remove all the item from the association table.
   */
  void removeAll();

  /*! Resize the table.
   * @param nSize new table size.
   */
  void resize( SL nSize );

  /*! Get the size of the table.
   * @return size of the table.
   */
  SL size() const;

  /*! Get the number of association in the table.
   * @return number of association in the table.
   */
  SL count() const;

  /*! Test if the table is empty.
   * @return true if there is no association in the table, false otherwise.
   */
  bool isEmpty() const;

#ifdef _SUPPORT_TEMPLATE_FRIEND
private:
#endif
  struct ELcAsso
  {
    ELcAsso *_pNextByKey;
    ELcAsso *_pNextByItem;
    Key _key;
    Item _item;
  };

  EArray<ELcAsso *> _aAssoByKey;
  EArray<ELcAsso *> _aAssoByItem;

  SL _nNbAsso;

protected:
  virtual UL doGetKey( KeyArg key ) const =0;
  virtual UL doGetKeyItem( ItemArg key ) const =0;
  virtual bool doCompareKey( KeyArg key1, KeyArg key2 ) const =0;
  virtual bool doCompareItem( ItemArg key1, ItemArg key2 ) const =0;
  virtual void doFreeAsso( Key &key, Item &item ) {}
};


// Includes:
// #####################Last change: 27/03/98 21h49mn#########################

#include "EHashBiBaseItT.h"

// 
// #####################Last change: 27/03/98 21h49mn#########################

template<class Key, class Item, class KeyArg, class ItemArg>
EHashBiBaseT<Key,Item,KeyArg,ItemArg>::EHashBiBaseT( SL nSize )
{
  EASSERT( nSize > 0 );
  _nNbAsso = 0;
  resize( nSize );
}


template<class Key, class Item, class KeyArg, class ItemArg>
EHashBiBaseT<Key,Item,KeyArg,ItemArg>::~EHashBiBaseT()
{
  removeAll();
}


template<class Key, class Item, class KeyArg, class ItemArg>
Item *
EHashBiBaseT<Key,Item,KeyArg,ItemArg>::lookUp( KeyArg key )
{
  SL nIndex = doGetKey( key ) % _aAssoByKey.count();
  ELcAsso *pAsso = _aAssoByKey[ nIndex ];
  while ( pAsso )
  {
    if ( doCompareKey( pAsso->_key, key ) )
      return &pAsso->_item;
    pAsso = pAsso->_pNextByKey;
  }
  return NULL;
}


template<class Key, class Item, class KeyArg, class ItemArg>
const Item *
EHashBiBaseT<Key,Item,KeyArg,ItemArg>::lookUp( KeyArg key ) const
{
  SL nIndex = doGetKey( key ) % _aAssoByKey.count();
  ELcAsso *pAsso = _aAssoByKey[ nIndex ];
  while ( pAsso )
  {
    if ( doCompareKey( pAsso->_key, key ) )
      return &pAsso->_item;
    pAsso = pAsso->_pNextByKey;
  }
  return NULL;
}


template<class Key, class Item, class KeyArg, class ItemArg>
Key *
EHashBiBaseT<Key,Item,KeyArg,ItemArg>::lookUpByItem( ItemArg item )
{
  SL nIndex = doGetKeyItem( item ) % _aAssoByItem.count();
  ELcAsso *pAsso = _aAssoByItem[ nIndex ];
  while ( pAsso )
  {
    if ( doCompareItem( pAsso->_item, item ) )
      return &pAsso->_item;
    pAsso = pAsso->_pNextByItem;
  }
  return NULL;
}


template<class Key, class Item, class KeyArg, class ItemArg>
const Key *
EHashBiBaseT<Key,Item,KeyArg,ItemArg>::lookUpByItem( ItemArg item ) const
{
  SL nIndex = doGetKeyItem( item ) % _aAssoByItem.count();
  ELcAsso *pAsso = _aAssoByItem[ nIndex ];
  while ( pAsso )
  {
    if ( doCompareItem( pAsso->_item, item ) )
      return &pAsso->_key;
    pAsso = pAsso->_pNextByItem;
  }
  return NULL;
}


template<class Key, class Item, class KeyArg, class ItemArg>
const Key *
EHashBiBaseT<Key,Item,KeyArg,ItemArg>::lookUpKey( KeyArg key ) const
{
  SL nIndex = doGetKey( key ) % _aAssoKey.count();
  ELcAsso *pAsso = _aAssoByKey[ nIndex ];
  while ( pAsso )
  {
    if ( doCompareKey( pAsso->_key, key ) )
      return &pAsso->_key;
    pAsso = pAsso->_pNextByKey;
  }
  return NULL;
}


template<class Key, class Item, class KeyArg, class ItemArg>
const Item *
EHashBiBaseT<Key,Item,KeyArg,ItemArg>::lookUpItem( ItemArg item ) const
{
  SL nIndex = doGetKeyItem( item ) % _aAssoByItem.count();
  ELcAsso *pAsso = _aAssoByItem[ nIndex ];
  while ( pAsso )
  {
    if ( doCompareItem( pAsso->_item, item ) )
      return &pAsso->_item;
    pAsso = pAsso->_pNextByItem;
  }
  return NULL;
}


template<class Key, class Item, class KeyArg, class ItemArg>
Item& EHashBiBaseT<Key,Item,KeyArg,ItemArg>::operator[]( KeyArg key )
{
  Item *pItem = lookUp( key );
  EASSERT( pItem );
  return *pItem;
}

/*
template<class Key, class Item, class KeyArg, class ItemArg>
const Item& EHashBiBaseT<Key,Item,KeyArg,ItemArg>::operator[]( const Key key ) const
{
  const Item *pItem = lookUp( key );
  EASSERT( pItem );
  return *pItem;
}
*/

template<class Key, class Item, class KeyArg, class ItemArg>
void 
EHashBiBaseT<Key,Item,KeyArg,ItemArg>::add( KeyArg key,
                                            ItemArg item )
{
  EASSERT( _aAssoByKey.count() > 0 );
  EASSERT( _aAssoByItem.count() > 0 );

  SL nIndexKey = doGetKey( key ) % _aAssoByKey.count();
  SL nIndexItem = doGetKeyItem( item ) % _aAssoByItem.count();

  ELcAsso *pAsso = new ELcAsso;
  pAsso->_key = key;
  pAsso->_item = item;
  pAsso->_pNextByKey = _aAssoByKey[ nIndexKey ];
  pAsso->_pNextByItem = _aAssoByItem[ nIndexItem ];
  _aAssoByKey[ nIndexKey ] = pAsso;
  _aAssoByItem[ nIndexItem ] = pAsso;
  _nNbAsso++;
}


template<class Key, class Item, class KeyArg, class ItemArg>
void 
EHashBiBaseT<Key,Item,KeyArg,ItemArg>::remove( KeyArg key )
{
  EASSERT( _aAsso.count() > 0 );

  SL nIndexKey = doGetKey( key ) % _aAssoByKey.count();
  ELcAsso *pAsso = _aAssoByKey[ nIndexKey ];
  ELcAsso **ppPrevAsso = &_aAssoByKey[ nIndexKey ];
  while ( pAsso )
  {
    if ( doCompareKey( pAsso->_key, key ) )  // found !
    {
      *ppPrevAsso = pAsso->_pNextByKey;

      ELcAsso pAssoThis = pAsso;      
      SL nIndexItem = doGetKeyItem( item ) % _aAssoItem.count();
      
      pAsso = _aAssoByItem[ nIndexItem ];
      ppPrevAsso = &_aAssoByItem[ nIndexItem ];
      while ( pAsso != pAssoThis )
      {
        ppPrevAsso = &pAsso->_pNextByItem;
        pAsso = pAsso->_pNextByItem;
        EASSERT( false );   // serious problem there. Found by key but not by item...
      }

      *ppPrevAsso = pAsso->_pNextByItem;
      doFreeAsso( pAssoThis->_key, pAssoThis->_item );
      delete pAssoThis;
      --_nNbAsso;
      return;
    }

    ppPrevAsso = &pAsso->_pNextByKey;
    pAsso = pAsso->_pNextByKey;
  }
  EASSERT( pAsso );
}


template<class Key, class Item, class KeyArg, class ItemArg>
void EHashBiBaseT<Key,Item,KeyArg,ItemArg>::resize( SL nSize )
{
  EDynArray<ELcAsso *> aAssoByKey2;
  EDynArray<ELcAsso *> aAssoByItem2;
  SL nCpt;
  ELcAsso *pNull = NULL;

  aAssoByKey2.resize( nSize );
  aAssoByItem2.resize( nSize );
  aAssoByKey2.fill( &pNull );
  aAssoByItem2.fill( &pNull );

  for ( nCpt = 0; nCpt < size(); ++nCpt )
  {
    ELcAsso *pAsso = _aAssoByKey[ nCpt ];
    while ( pAsso )
    {
      ELcAsso *pNextAsso = pAsso->_pNextByKey;
      SL nIndexKey = doGetKey( pAsso->_key ) % nSize;    // compute new index

      pAsso->_pNextByKey = aAssoByKey2[ nIndexKey ];        // add to head of lc
      aAssoByKey2[ nIndexKey ] = pAsso;

      pAsso = pNextAsso;
    }

    pAsso = _aAssoByItem[ nCpt ];
    while ( pAsso )
    {
      ELcAsso *pNextAsso = pAsso->_pNextByItem;
      SL nIndexItem = doGetKeyItem( pAsso->_item ) % nSize;    // compute new index

      pAsso->_pNextByItem = aAssoByItem2[ nIndexItem ];        // add to head of lc
      aAssoByItem2[ nIndexItem ] = pAsso;

      pAsso = pNextAsso;
    }
  }

  _aAssoByKey = aAssoByKey2;
  _aAssoByItem = aAssoByItem2;
}


template<class Key, class Item, class KeyArg, class ItemArg>
inline SL 
EHashBiBaseT<Key,Item,KeyArg,ItemArg>::size() const
{
  EASSERT( _aAssoByKey.count() == _aAssoByItem.count() );
  return _aAssoByKey.count();
}


template<class Key, class Item, class KeyArg, class ItemArg>
inline SL 
EHashBiBaseT<Key,Item,KeyArg,ItemArg>::count() const
{
  return _nNbAsso;
}


template<class Key, class Item, class KeyArg, class ItemArg>
void 
EHashBiBaseT<Key,Item,KeyArg,ItemArg>::removeAll()
{
  SL nIndex;

  for ( nIndex = size() -1; nIndex >= 0; --nIndex )
  {
    ELcAsso *pAsso = _aAssoByKey[ nIndex ];
    while ( pAsso )
    {
      ELcAsso *pNextAsso = pAsso->_pNextByKey;
      doFreeAsso( pAsso->_key, pAsso->_item );
      delete pAsso;
      pAsso = pNextAsso;
    }
    _aAssoByKey[ nIndex ] = NULL;
    _aAssoByItem[ nIndex ] = NULL;
  }
  _nNbAsso = 0;
}


template<class Key, class Item, class KeyArg, class ItemArg>
inline bool 
EHashBiBaseT<Key,Item,KeyArg,ItemArg>::isEmpty() const
{
  return count() == 0;
}

#endif // !defined(AFX_EHASHBIBASET_H__BF9A2C73_750D_11D3_BE99_0040053AD648__INCLUDED_)
