// EObjectContext.h: interface for the EObjectContext class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EOBJECTCONTEXT_H__04B1E883_6925_11D2_AEB1_A9DB09C4B161__INCLUDED_)
#define AFX_EOBJECTCONTEXT_H__04B1E883_6925_11D2_AEB1_A9DB09C4B161__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "EDataStr.hpp"

class EClassId;

class EObjectContext  
{
public:
  /*! Default constructor.
   */
  EObjectContext();
	
  /*! Constructs an object storage context.
   * @param nVersion Class storage format version number.
   * @param clsidObject Class id of object to save/load.
   */
  EObjectContext( UL nVersion,
                  const EClassId &clsidObject );
	
  /*! Constructs an object storage context.
   * @param clsidObject Class id of object to save/load.
   */
  EObjectContext( const EClassId &clsidObject );

  /*! Destructor.
   */
  ~EObjectContext() {}

  /*! Set object storage version.
   * @param nVersion Class storage format version number.
   */
  void setVersion( UL nVersion );

  /*! Set object storage class id.
   * @param clsidObject Class id of object to save/load.
   */
  void setClass( const EClassId &clsidObject );

  /*! Get object storage format version.
   * @return Object storage format version.
   */
  UL version() const;
  
  /*! Get object class id.
   * @return Get object class id.
   */
  const EClassId &clsid() const;

  EDataStream &operator <<( EDataStream &s ) const;
  EDataStream &operator >>( EDataStream &s );

private:
  UL _nVersion;
  const EClassId *_pClsidObject;
};

EDataStream &operator <<( EDataStream &s,
                          const EObjectContext &ctx );

EDataStream &operator >>( EDataStream &s,
                          EObjectContext &ctx );

inline 
EObjectContext::EObjectContext()
{
  _nVersion =0;
  _pClsidObject =NULL;
}


inline 
EObjectContext::EObjectContext( UL nVersion,
                                const EClassId &clsidObject )
{
  _nVersion = nVersion;
  _pClsidObject = &clsidObject;
}


inline void 
EObjectContext::setVersion( UL nVersion )
{
  _nVersion = nVersion;
}


inline void 
EObjectContext::setClass( const EClassId &clsidObject )
{
  _pClsidObject = &clsidObject;
  EASSERT( _pClsidObject );   // must be defined !
}


inline UL 
EObjectContext::version() const
{
  return _nVersion;
}


inline const EClassId &
EObjectContext::clsid() const
{
  EASSERT( _pClsidObject );     // Class id not set !!!
  return *_pClsidObject;
}


#endif // !defined(AFX_EOBJECTCONTEXT_H__04B1E883_6925_11D2_AEB1_A9DB09C4B161__INCLUDED_)
