// ERef.h: interface for the ERef class.
//
//////////////////////////////////////////////////////////////////////

/* HST:
 * 18/10/1998: -BugFix: Forgot override operator =(const EBaseRef &).
 * 19/10/1998: - Add cast operator : (Type*).
 * 19/10/1998: - Add == operator.
 */

#if !defined(AFX_EREF_H__A1E5DBDF_480B_11D2_AE43_F8FAF6E5B561__INCLUDED_)
#define AFX_EREF_H__A1E5DBDF_480B_11D2_AE43_F8FAF6E5B561__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "EBaseRef.h"

// Build the name of the reference class.
#define E_REFNAME( ClassName ) ClassName##Ref

// Declare a new reference class.
#define EDECL_REF( Type ) \
  class Type; \
  typedef ERef<##Type##> E_REFNAME( Type )


/*! \brief This class provides a mean to acces objects using reference counting.
 *
 * @param Type Type of object referenced. \a Type must be derived from Type.
 */
template<class Type>
class ERef : public EBaseRef
{
public:
  typedef ERef<Type> ThisRefType;
  typedef Type ThisType;

  static const ERef<Type> nullObject;

	/*! Constructs a \c NULL reference.
   */
  ERef() {}
  
  /*! Constructs a reference from a pointer to an object.
   * @param p Object that is referenced.
   */
	ERef( Type *p );

  /*! Constructs a reference from another reference.
   * @param ref Reference to copy.
   */
  ERef( const ERef<Type> &ref );

  /*! Releases the reference to the object.
   */
  virtual ~ERef() {}
#if 0
	Type& operator*();

	Type** operator&();
#endif
	Type* operator->() const;

	ERef<Type> &operator=( Type *p );

	ERef<Type> &operator=( const ERef<Type> &ref );

  operator Type *() const;

  /*! Tests if two references are equal.
   * @return \c true if the two reference are equal, \c false otherwise.
   */
  bool operator ==( const ERef<Type> &ref ) const;
};



template<class Type>
const ERef<Type> ERef<Type>::nullObject;


template<class Type>
inline 
ERef<Type>::ERef( Type *p ) :
    EBaseRef( p )
{
}


template<class Type>
inline 
ERef<Type>::ERef( const ERef<Type>& ref ) :
    EBaseRef( ref )
{
}


#if 0
  template<class Type>
  inline Type &
  ERef<Type>::operator*()
  {
    return (Type &)EBaseRef::operator*();
  }


  template<class Type>
  inline Type** 
  ERef<Type>::operator &() 
  { 
    return (Type**)EBaseRef::operator&();
  }
#endif


template<class Type>
inline Type *
ERef<Type>::operator ->() const
{ 
  return (Type *)EBaseRef::operator->();
}


template<class Type>
inline ERef<Type> &
ERef<Type>::operator =( Type *p )
{
  if ( _p )
    _p->decRef();
  _p = p;
  if ( p )
    p->incRef();
  return *this;
}


template<class Type>
inline  ERef<Type> &
ERef<Type>::operator=( const ERef<Type>& ref )
{
  EBaseRef::operator=( ref );
  return *this;
}


template<class Type>
inline
ERef<Type>::operator Type *() const
{
  return (Type *)_p;
}


template<class Type>
inline bool 
ERef<Type>::operator ==( const ERef<Type>& ref ) const
{
  return _p == ref._p;
}


#endif // !defined(AFX_EREF_H__A1E5DBDF_480B_11D2_AE43_F8FAF6E5B561__INCLUDED_)
