// ERefObj.h: interface for the ERefObj class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EREFOBJ_H__A1E5DBE0_480B_11D2_AE43_F8FAF6E5B561__INCLUDED_)
#define AFX_EREFOBJ_H__A1E5DBE0_480B_11D2_AE43_F8FAF6E5B561__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000



/*! This class provides reference counting for any objects.
 *  Reference on object must be stored using FarRef variable.
 */
class ERefObj  
{
public:
	ERefObj();
  virtual ~ERefObj() {}

  /*! Add a new reference on the object.
   */
  void incRef();

  /*! Remove a reference on the object.
   * When there is no more reference to the object, destroy() is called.
   */
  void decRef();

  /*! Get the reference count.
   * @return Reference count.
   */
  UL refCount() const;

protected:
  /*! Destroy the object.
   *
   * The default implementation is to do a \c delete \c this.
   */
  virtual void destroy();

private:
  /*! Number of reference on the object.
   */
  UL _nRefCount;
};

inline 
ERefObj::ERefObj()
{
  _nRefCount =0;
}


inline void 
ERefObj::incRef()
{
  EASSERT( _nRefCount < UL_MAX );
  _nRefCount++;
}


inline void 
ERefObj::decRef()
{
  EASSERT( _nRefCount > 0 );    // major bug in reference count if failed !!!
  if ( --_nRefCount == 0 )        // free the object !
    destroy();
}

inline UL 
ERefObj::refCount() const
{
  return _nRefCount;
}


#endif // !defined(AFX_EREFOBJ_H__A1E5DBE0_480B_11D2_AE43_F8FAF6E5B561__INCLUDED_)
