// EStoreObjectPrp.cpp: implementation of the EStoreObjectPrp class.
//
//////////////////////////////////////////////////////////////////////

#include "usuel.hpp"
#include "EString.hpp"
#include "EStoreOtPrp.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

EStoreOtPrp::EStoreOtPrp( EStorePrpType esptpPrp,
                                  const EString &strValue,
                                  bool bIsEncoding ) :
    _esptpPrp( esptpPrp ),
    _pvtpValue( pvtpNone ),
    _strValEnc( strValue )
{
  if ( bIsEncoding )    // set new prp value
    setValString( strValue );
}


EStoreOtPrp::EStoreOtPrp( EStorePrpType esptpPrp,
                                  SL nValue ) :
    _esptpPrp( esptpPrp )
{
   setValLong( nValue );
}


EStoreOtPrp::EStoreOtPrp( EStorePrpType esptpPrp,
                                  UL ulValue ) :
    _esptpPrp( esptpPrp )
{
   setValULong( ulValue );
}


EStoreOtPrp::EStoreOtPrp( EStorePrpType esptpPrp,
                                  double flValue ) :
    _esptpPrp( esptpPrp )
{
   setValDouble( flValue );
}


EStoreOtPrp::EStoreOtPrp( EStorePrpType esptpPrp ) :
    _esptpPrp( esptpPrp ),
    _pvtpValue( pvtpNone )
{
}


EStoreOtPrp::~EStoreOtPrp()
{
}


bool 
EStoreOtPrp::isValid()
{
  if ( _strValEnc.isNull() )
    return false;

  TCHAR tcHead = _strValEnc[ 0 ];
  if ( tcHead == _T( '#' ) )    // binary
  {
    setValueType( pvtpBinary );
    return isValidBinary();
  }
  
  if ( tcHead == _T( '"' ) )    // string
  {
    setValueType( pvtpString );
    return isValidString();
  }

                                // number
  setValueType( pvtpNumber );
  return isValidNumber();
}


bool 
EStoreOtPrp::isValidString()
{
  const TCHAR tcQuote = _T( '"' );
  const TCHAR tcEscape = _T( '#' );   // char for escape seqence #r #n #xnn
  if ( _strValEnc.length() < 2 )
    return false;

  if ( _strValEnc[ 0 ] != tcQuote  ||
       _strValEnc.lastChar() != tcQuote )
    return false;

  SL nPos =1;   // current position
  SL nPosEnd = _strValEnc.length() -1;    // position we stop at (before the last '"').
  while ( nPos < nPosEnd )
  {
    TCHAR tcChar = _strValEnc[ nPos++ ];
    if ( tcChar == tcEscape )   // escape sequence
    {
      if ( nPos >= nPosEnd )
        return false;

      tcChar = _strValEnc[ nPos++ ];      // escape sequence type ?
      if ( tcChar == _T( 'x' ) )             // hexa code
      {
        EString strHexa;
        if ( nPos +1>= nPosEnd )
          return false;

        strHexa += _strValEnc[ nPos++ ];
        strHexa += _strValEnc[ nPos++ ];

        if ( !strHexa.isHexaInt() )               // check if its an hexa
          return false;
      }
      else if ( tcChar != _T( 'r' )  &&           // check if cr, nl, or tab
                tcChar != _T( 'n' )  && 
                tcChar != _T( 't' ) )
      {
        return false;
      }
    }
  }

  return true;
}


bool 
EStoreOtPrp::isValidBinary()
{
  EString strBinary = _strValEnc.mid( 1, -1 );
  if ( !strBinary.isNull() ) 
  {
    return (strBinary.length() & 1 ) == 0  &&       // length if even (2hexa for 1byte)
           strBinary.isHexaInt();
  }
  return true;          // Empty binary stream...
}


bool 
EStoreOtPrp::isValidNumber()
{
  const TCHAR *tcszHexPrefix = _T( "0x" );

  EString strPrefix = _strValEnc.left( 2 );
  if ( !strPrefix.icmp( tcszHexPrefix ) )     // hexa number
  {
    EString strHexa = _strValEnc.mid( 2, -1 );
    return strHexa.isHexaInt();
  }

  // Check if double
  TCHAR *tcszEnd;
  _tcstod( _strValEnc.datas(),
           &tcszEnd );
  if ( tcszEnd  &&  *tcszEnd != 0 )     // error while parsing double
    return false;

  return true;
}


SL 
EStoreOtPrp::valLong()
{
  return (SL)valDouble();
}


UL 
EStoreOtPrp::valULong()
{
  return (UL)valLong();
}


double 
EStoreOtPrp::valDouble()
{
  const TCHAR *tcszHexPrefix = _T( "0x" );

  EString strPrefix = _strValEnc.left( 2 );
  if ( !strPrefix.icmp( tcszHexPrefix ) )     // hexa number
  {
    EString strHexa = _strValEnc.mid( 2, -1 );
    return (double)strHexa.valHexaInt();
  }

  return _tcstod( _strValEnc.datas(), NULL );
}


void 
EStoreOtPrp::valString( EString &strValue )
{
  const TCHAR tcQuote = _T( '"' );
  const TCHAR tcEscape = _T( '#' );   // char for escape seqence #r #n #xnn

  strValue.setNull();

  SL nPos =1;   // current position
  SL nPosEnd = _strValEnc.length() -1;    // position we stop at (before the last '"').
  while ( nPos < nPosEnd )
  {
    TCHAR tcChar = _strValEnc[ nPos++ ];
    if ( tcChar == tcEscape )   // escape sequence
    {
      tcChar = _strValEnc[ nPos++ ];      // escape sequence type ?
      if ( tcChar == _T( 'x' ) )             // hexa code
      {
        EString strHexa;
        strHexa += _strValEnc[ nPos++ ];
        strHexa += _strValEnc[ nPos++ ];

        SL nHexa = strHexa.valHexaInt();
        strValue += (TCHAR)nHexa;
      }
      else if ( tcChar != _T( 'r' ) )
        strValue += _T( '\r' );
      else if ( tcChar != _T( 'n' ) )
        strValue += _T( '\n' );
      else if ( tcChar != _T( 't' ) )
        strValue += _T( '\t' );
    }
    else
      strValue += tcChar;
  }
}


void 
EStoreOtPrp::valBinary()
{
  EASSERT( false );

  // rework class EBuffer...
}



void 
EStoreOtPrp::setValLong( SL nValue )
{
  setValueType( pvtpNumber );
  _strValEnc.format( "%d", nValue );
}


void 
EStoreOtPrp::setValULong( UL ulValue )
{
  setValueType( pvtpNumber );
  _strValEnc.format( "%u", ulValue );
}


void 
EStoreOtPrp::setValDouble( double flValue )
{
  setValueType( pvtpNumber );
  _strValEnc.format( "%g", flValue );
}


void 
EStoreOtPrp::setValString( const EString &strValue )
{
  setValueType( pvtpString );
  _strValEnc = '"';
  SL nPos;
  for ( nPos =0; nPos < strValue.length(); ++nPos )
  {
    TCHAR tcChar = strValue[ nPos ];
    if ( tcChar >= 32 )
      _strValEnc += tcChar;
    else
    {
      _strValEnc += _T( '#' );

      if ( tcChar == _T( '\t' ) )
        _strValEnc += 9;
      else if ( tcChar == _T( '\r' ) )
        _strValEnc += 13;
      else if ( tcChar == _T( '\n' ) )
        _strValEnc += 10;
      else
      {
        _strValEnc += _T( 'x' );
        EString strHexa;
        strHexa.setHexa( tcChar, NULL, 2 );
        _strValEnc += strHexa;
      }
    }
  }

  _strValEnc += '"';
}


void 
EStoreOtPrp::setValBinary()
{
  setValueType( pvtpBinary );
  EASSERT( false );
}
