// ###########################################################################
// File: EBITARR.CPP
// (c)Copyright 03/1998, Lepilleur Baptiste (Aka Elyon from Zuul Design)
// ###########################################################################

#include <string.h>
#include <stdlib.h>
#include "usuel.hpp"
#include "ebitarr.hpp"

/** Default Constructor.
  */
EBitArray::EBitArray()
{
  init();
}


/// initialize members.
void EBitArray::init()
{
  _nLength = 0;
  _aBits = ENIL;
}


/** Set a slice of bit in the array.
  * @param nIndex index of the first bit of the slice to set.
  * @param nLength number of bits in the slice.
  * @param bSet if true then the bits are set, otherwise they are unset.
  */
void EBitArray::setSlice( SL nIndex, SL nLength, bool bSet )
{
  assert( nIndex >= 0 );
  assert( nLength >= 0 );
  assert( nIndex + nLength <= _nLength );
  SL nIndexEnd = nIndex + nLength -1;
  SL nStart = nIndex /8;
  SL nEnd = nIndexEnd /8;
  UB nLeftMask = 0xff <<  (nIndex & 7);   // mask to keep bits on left
  UB nRightMask = 0xff << (nIndex & 7);   // mask to keep bits on right

  if ( nStart == nEnd )
  {
    UB nMask = nLeftMask | nRightMask;
    _aBits[ nStart ] &= nMask;
    if ( bSet )
      _aBits[ nStart ] |= ~nMask;
  }
  else
  {
    _aBits[ nStart ] &= nLeftMask;
    _aBits[ nEnd ] &= nRightMask;
    if ( bSet )
    {
      _aBits[ nStart ] |= ~nLeftMask;
      _aBits[ nEnd ] |= ~nRightMask;
    }

    if ( nEnd - nStart > 1 )
    {
      memset( &_aBits[ nStart +1 ],
              bSet ? 0xff : 0,
              sizeof(UL)  * ( nEnd -nStart -1 ) );
    }
  }
}


/** Resize the array.
  * @param nSize new size of the array, in bits.
  * @param bClear if the array is enlarged and bClear is true the new bits
  *        are unset, otherwise they are undetermined.
  */
void EBitArray::resize( SL nSize, bool bSet )
{
  assert( nSize >= 0 );
  if ( nSize == _nLength )
    return;

  UB *aNewBits = (UB*)erealloc( _aBits, sizeof(UB)* ((nSize + 7) /8) );
  _aBits = aNewBits;
  SL nOldLength = _nLength;
  _nLength = nSize;

  if ( bSet  &&  nSize > nOldLength )
    setSlice( _nLength, nSize -_nLength, false );
}


/** Copy another bit array.
  * @param bits array to copy.
  */
void EBitArray::copy( const EBitArray& bits )
{
  resize( bits.count(), false );
  memcpy( _aBits, bits._aBits, sizeof(UB)*bufferSize() );
}

/** Assignment operator.
  * @param bits array to copy.
  */
EBitArray &EBitArray::operator =( const EBitArray &bits )
{
  copy( bits );
  return *this;
}



// ########################### EOF EBITARR.CPP ###############################
// ########################### EOF EBITARR.CPP ###############################
// ########################### EOF EBITARR.CPP ###############################

