// ###########################################################################
// File: EBUFFER.HPP
// (c)Copyright 08/1997, Lepilleur Baptiste (Aka Elyon from Zuul Design)
// ###########################################################################


#ifndef EBUFFER_HPP_INCLUDED


// Includes:
// #####################Last change: 22/08/97  8h03mn#########################

#include "EArray.h"
#include "eiodevic.hpp"

// Constantes:
// #####################Last change: 22/08/97  8h03mn#########################


// Macros:
// #####################Last change: 22/08/97  8h03mn#########################


// Types:
// #####################Last change: 22/08/97  8h03mn#########################


class EBuffer : public EIODevice
{
public:
    typedef EBinDynArrayT<UB> TypeArray;

    /** Constructs a buffer.
     * Constructs a buffer that can hold at least \a size elements.
     * @param size initial size of the buffer.
     */
    EBuffer( int size =0 );

    /** Constructs a buffer.
     * Constructs a buffer that used the dynamic array \a a as buffer.
     * If \a own is true then the array is deleted when the buffer is
     * destroyed.
     * @param size initial size of the buffer.
     * @param own the array is destroyed on destruction is its true.
     */
    EBuffer( TypeArray *a, bool own =false );

    /// Destructor
    ~EBuffer();


    /** Open the device.
     * Open the device using mode \a _mode.
     * @param _mode mode use to open the device
     */
    void open( int _mode );

    /** Close the device.
     */
    void close();

    /** Flush the device.
     */
    void flush();

    /** Return the size of the device.
     * @return size of the device.
     */
    SL size() const;

    /** Return the index of the device.
     * @return index of the device.
     * @exception EXCIODevice on error.
     */
    SL at() const;

    /** Set the position of the device.
     * Set the position of the device (index); to \a pos.
     * @param pos new position of the index.
     * @exception EXCIODevice on error.
     */
    void at( SL pos);

    /** Test if the device is at the end.
     * @return true if the device is at the end, false otherwise.
     */
    bool atEnd() const;

    /** Read a block of byte from the device.
     * Read \a len Byte from the device and store them in the buffer \a data.
     * @param data buffer where data are stored.
     * @param len number of bytes of data to read.
     * @return number of byte of data actually read.
     * @exception EXCIODevice is case of grave error.
     */
    SL readBlock( void *data, SL len);

    /** Write a block of byte on the device.
     * Write \a len Byte of buffer \a data on the device.
     * @param data buffer where data are read.
     * @param len number of bytes of data to write.
     * @exception EXCIODevice is case of grave error.
     */
    void writeBlock( const void *data, SL len );

    /** Read a byte from the stream.
     * @return byte read.
     * @exception EIODevice on error
     */
    UB getByte();

    /** Write a Byte on the stream.
     * @param c byte to write.
     * @exception EIODevice on error.
     */
    void putByte( UB c );

    /** Push-back a byte on the stream.
     * @param c byte to push-back.
     * @exception EIODevice on error.
     */
    void ungetByte( UB c );

private:

    /// buffer.
    TypeArray *mBuffer;

    /// buffer is ours ?
    bool mOwnBuffer;

};


// Globales:
// #####################Last change: 22/08/97  8h03mn#########################


// Prototypes:
// #####################Last change: 22/08/97  8h03mn#########################



#define EBUFFER_HPP_INCLUDED
#endif


// ########################### EOF EBUFFER.HPP ###############################
// ########################### EOF EBUFFER.HPP ###############################
// ########################### EOF EBUFFER.HPP ###############################

