// ###########################################################################
// File: EDATASTR.HPP
// (c)Copyright 08/1997, Lepilleur Baptiste (Aka Elyon from Zuul Design)
// ###########################################################################


#ifndef EDATASTR_HPP_INCLUDED


// Includes:
// #####################Last change: 22/08/97 21h31mn#########################

#include "estream.hpp"

// Constantes:
// #####################Last change: 22/08/97 21h31mn#########################


// Macros:
// #####################Last change: 22/08/97 21h31mn#########################

// Define this to disable EString support.
//#define EDATASTREAM_NOSUPPORT_ESTRING

// Types:
// #####################Last change: 22/08/97 21h31mn#########################

#ifndef EDATASTREAM_NOSUPPORT_ESTRING
class EString;
#endif


class EDataStream : public EStream
{
public:
    /** Constructs a data stream with a device.
     * Constructs a data stream for device \a device.
     * @param pDevice device used by the data stream.
     */
    EDataStream( EIODevice *pDevice );

    /** Read a block.
     * @param pData data to read.
     * @param nLen length of data to read in byte.
     */
    void readBlock( void *pData, SL nLen );

    /** Write a block.
     * @param pData data to write.
     * @param nLen length of data to write in byte.
     */
    void writeBlock( const void *pData, SL nLen );

    /** Get a byte from the stream.
     * @return byte read.
     */
    UB getByte();

    /** Write a byte in the stream.
     * @param v byte to write.
     */
    void putByte( UB v);

    EDataStream& operator >>(SB& v);
    EDataStream& operator >>(SW& v);
    EDataStream& operator >>(SL& v);
    EDataStream& operator >>(UB& v);
    EDataStream& operator >>(UW& v);
    EDataStream& operator >>(UL& v);
    EDataStream& operator >>(float& v);
    EDataStream& operator >>(double& v);
    EDataStream& operator >>(char *& v);
    EDataStream& operator <<(SB v);
    EDataStream& operator <<(SW v);
    EDataStream& operator <<(SL v);
    EDataStream& operator <<(UB v);
    EDataStream& operator <<(UW v);
    EDataStream& operator <<(UL v);
    EDataStream& operator <<(float v);
    EDataStream& operator <<(double v);
    EDataStream& operator <<(const char * v);

#ifdef EBOOL_IS_BASE_TYPE
    EDataStream& operator >>( bool &v );
    EDataStream& operator <<( bool v );
#endif

    /*! Read a c-string from the stream.
     * @param s On return, allocated array which contains the c-string. Notes that you
     *          must destroy that array after use.
     */
    EDataStream &readCStr( char * &s);

    /*! Write a c-string in the c-string format.
     * @param s C-String to write.
     */
    EDataStream &writeCStr( const char *s );

#ifndef EDATASTREAM_NOSUPPORT_ESTRING
    /*! Read a c-string from the stream.
     * @param s On return, read string.
     */
    EDataStream &readCStr( EString &s);
#endif

};


// inlines:
// #####################Last change: 22/08/97 21h31mn#########################


/** Get a byte from the stream.
  * @return byte read.
  */
inline UB EDataStream::getByte()
{
#ifdef PRINTABLE_STREAM
  if ( isPrintable() )
  {
    SB c;
    getPrintableByte( c );
    return (UB)c;
  }
#endif
  return device()->getByte();
}

/** Write a byte in the stream.
  * @param v byte to write.
  */
inline void EDataStream::putByte( UB v )
{
#ifdef PRINTABLE_STREAM
  if ( isPrintable() )
  {
    putPrintableByte( v );
    return;
  }
#endif
  device()->putByte( v );
}

inline EDataStream& EDataStream::operator >>(UB& v)
{
  return *this >> (SB&)v;
}

inline EDataStream& EDataStream::operator >>(UW& v)
{
  return *this >> (SW&)v;
}

inline EDataStream& EDataStream::operator >>(UL& v)
{
  return *this >> (SL&)v;
}

inline EDataStream& EDataStream::operator <<(UB v)
{
  return *this << (SB)v;
}

inline EDataStream& EDataStream::operator <<(UW v)
{
  return *this << (SW)v;
}

inline EDataStream& EDataStream::operator <<(UL v)
{
  return *this << (SL)v;
}

inline EDataStream& EDataStream::operator >>(SB& v)
{
#ifdef PRINTABLE_STREAM
  if ( isPrintable() )
  {
    getPrintableByte( v );
    return *this;
  }
#endif
  v = getByte();
  return *this;
}

inline EDataStream& EDataStream::operator <<(SB v)
{
#ifdef PRINTABLE_STREAM
  if ( isPrintable() )
  {
    putPrintableByte( v );
    return *this;
  }
#endif

  putByte( v );
  return *this;
}



#ifdef EBOOL_IS_BASE_TYPE

inline EDataStream& 
EDataStream::operator >>( bool &v )
{
  return *this >> (UB&)v;
}


inline EDataStream& 
EDataStream::operator <<( bool v )
{
  return *this << (UB)v;
}

#endif


// Prototypes:
// #####################Last change: 22/08/97 21h31mn#########################



#define EDATASTR_HPP_INCLUDED
#endif


// ########################### EOF EDATASTR.HPP ##############################
// ########################### EOF EDATASTR.HPP ##############################
// ########################### EOF EDATASTR.HPP ##############################

