// ###########################################################################
// File: EFILESTD.CPP
// (c)Copyright 10/1997, Lepilleur Baptiste (Aka Elyon from Zuul Design)
// version 0.21
// ###########################################################################

/* History:
 * 26/04/98: - Fix bug when included many time efilestd.h
 */

#include <io.h>
#include "usuel.hpp"
#include "efilestd.hpp"

// Default Constructor
EFileStd::EFileStd()
{
    init();
}

// Constructs a file that used stream \a f.
EFileStd::EFileStd( FILE *f )
{
  init();
  setStream( f );
}

// Destructor
EFileStd::~EFileStd()
{
  EIODevice::~EIODevice();
}

// Initialisation
void EFileStd::init()
{
    mFile =0;
    mLength =0;
    mIndex =0;
    resetInfo();
}


// set stream used.
void EFileStd::setStream( FILE *file )
{
  assert( file );
  mFile = file;
}


/** Open the file.
  * Open the file of name given on construction or with \l setName() using
  * mode \a _mode.
  * @param _mode mode used to open the file.
  * @see EIODevice::flags()
  * @exception EXCIODevice raised on error
  */
void EFileStd::open( int _mode )
{
  assert( !isOpen() );
//  assert( !fn.isNull() );
  FILE *f =mFile;
  init();
  mFile =f;
  setMode( _mode );

  assert( !isReadable()  ||  !isWritable() );
  setStatus( EIO_IsOpen );
/*
  const char *perm =0;
  bool retryIfFailed = false;

  if ( isReadWrite() )
  {
      if ( accessMode() & EIO_Truncate )
          perm = "w+b";
      else
      {
          perm = "r+b";
          retryIfFailed = true;
      }
  }
  else
  {
      if ( isReadable() )
          perm = "rb";
      else
          if ( isWritable() )
              perm = "wb";
  }

  f = fopen( (const char *)fn, perm);
  if ( !f  &&  retryIfFailed )
      f = fopen( (const char *)fn, "wb");    // file do not exist, create one
  if ( f )
  {
      fseek( f, 0, SEEK_END );
      mLength = ftell( f );
      fseek( f, 0, SEEK_SET );
      index =0;

      setStatus( EIO_IsOpen );
  }
  else
      raiseEXC( EXCIODevice( EXCIODevice::createError ) );
*/
}

// Close the file.
void EFileStd::close()
{
    assert( isOpen() );
    unsetStatus( EIO_IsOpen );
    init();
}

// Flush the file.
void EFileStd::flush()
{
    assert( isOpen() );
    fflush( mFile );
}

// Return the size of the file
SL EFileStd::size() const
{
  return 0;
}

/** Set the index of the file.
  * @param pos position of the index in the file.
  */
void EFileStd::at( SL pos )
{
  assert( isOpen() );

  if ( fseek( mFile, pos, SEEK_SET ) != 0 )
    raiseEXC( EXCIODevice( EXCIODevice::ioError ) );
}


/** Read a block of byte from the device.
  * Read \a len Byte from the device and store them in the buffer \a data.
  * @param data buffer where data are stored.
  * @param len number of bytes of data to read.
  * @return number of byte of data actually read.
  * @exception EXCIODevice is case of grave error.
  */
SL EFileStd::readBlock( void *data, SL len)
{
  assert( data != 0 );
  assert( isOpen() );
  assert( isReadable() );

  SL szRead;

  szRead = fread( data, 1, len, mFile );
  mIndex += szRead;

  return szRead;
}

/** Write a block of byte on the device.
  * Write \a len Byte of buffer \a data on the device.
  * @param data buffer where data are read.
  * @param len number of bytes of data to write.
  * @exception EXCIODevice is case of grave error.
  */
void EFileStd::writeBlock( const void *data, SL len )
{
  assert( data != 0 );
  assert( isOpen() );
  assert( isWritable() );

/*
  int szWrited = fwrite( data, 1, len, mFile );
  mIndex += szWrited;
  mLength = max( mLength, mIndex );      // update file length
  if ( szWrited != len )
      raiseEXC( EXCIODevice( EXCIODevice::ioError ) );
*/
  fwrite( data, 1, len, mFile );  // Always return 0
  mIndex += len;
  mLength = EMAX( mLength, mIndex );      // update file length
}

/** Read a byte from the stream.
  * @return byte read.
  * @exception EIODevice on error
  */
UB EFileStd::getByte()
{
  assert( isOpen() );
  assert( isReadable() );

  int c = getc( mFile );
  if ( c != EOF )
  {
    ++mIndex;
    return (UB)(c);
  }
  raiseEXC( EXCIODevice( EXCIODevice::ioError ) );
#ifdef _MSC_VER
  return 0;
#endif
}

/** Write a Byte on the stream.
  * @param c byte to write.
  * @exception EIODevice on error.
  */
void EFileStd::putByte( UB c )
{
  assert( isOpen() );
  assert( isWritable() );

  if ( fputc( c, mFile ) == EOF )
    raiseEXC( EXCIODevice( EXCIODevice::ioError ) );
  else
  {
    ++mIndex;
    mLength = EMAX( mLength, mIndex );
  }
}

/** Push-back a byte on the stream.
  * @param c byte to push-back.
  * @exception EIODevice on error.
  */
void EFileStd::ungetByte( UB c )
{
  assert( isOpen() );
  assert( isReadable() );
  ungetc( c, mFile );
  mIndex--;
}

