// ###########################################################################
// File: EXC.HPP
// (c)Copyright 12/1995, Lepilleur Baptiste (Aka Elyon from Zuul Design)
// Version 1.2
// ###########################################################################

#ifndef EXC_HPP_INCLUDED
#define EXC_HPP_INCLUDED

/** HST:
 * 30/09/97: changes to the new naming convention (m... for members) and
 *           set datas as private.
 * 29/03/98: - add macro: ETHROW,ECATCH, ETHROW_AGAIN, ETRY.
 *           - CLASS_NAME: class identication
 *           - bug: setContext, always return EXC* so exception type
 *                  was always EXC *  !!!!
 *                  Each exception class now have a method setContext
 *                  that return a reference of the proper type.
 *                  This method is implemented using EXC_TYPE macro.
 * 02/04/98: - add macro ECATCH_IGNORE, used to catch and ignore an exception.
 * 03/10/98: - add macro ECATCH_ANY, used to catch any exception (and usally rethrow them).
 *           - add macro ECATCH_IGNOREALL, used to catch any exception and ignore it.
 */

/* Notes:
 * - Change NEWEXC to ENEW_EXC
 * - Remove raiseEXC
 * - Remove NEWEXCD
 */

// Macros:
// #####################Last change: 30/09/97 19h10mn#########################

/** Raise the exception a and set the exception context.
 * Example of use:
 * raiseEXC( EXCEndOfFile( "toto.ssp" ) )
 */
//#define raiseEXC(a) throw (new a)->setContext(__LINE__, __FILE__)
#define raiseEXC(a) throw a.setContext(__LINE__, __FILE__)

/*! Raise an exception.
 *  @param a  Object that is thrown. Must inherits class EXC.
 */
#define ETHROW(a) throw a.setContext(__LINE__, __FILE__)

/*! Catch an exception.
 * @param type exception class.
 * @param var variable use for the exception.
 */
#define ECATCH(Type,var) catch( Type &var )

/*! Catch an exception of a given type and ignore it.
 */
#define ECATCH_IGNORE( Type ) catch( Type ) {}

/*! Catch any exception and ignore it.
 */
#define ECATCH_IGNOREALL catch( ... ) {}

/*! Catch an exception of a given type.
 */
#define ECATCH_TYPE( Type ) catch( Type )

/*! Catch any exception.
 */
#define ECATCH_ANY catch( ... )

/*! Rethrow the last catched exception.
 */
#define ETHROW_AGAIN throw

/*! Begin a exception catch block.
 */
#define ETRY try

/** Macro that must be present in all exception class.
 * @param Type class name.
 */
#define EXC_TYPE( Type )                                     \
    CLASS_NAME( Type );                                      \
    Type &setContext( UL nLine, const char *sSourceFile )      \
    {                                                        \
      EXC::setContext( nLine, sSourceFile );                   \
      return *this;                                          \
    }

/** Define a new exception.
 * Define a new exception class of classname \a name and information \a msg.
 * @param name name of the class.
 * @param msg information for the class.
 */
#define NEWEXC(name, msg)              \
    class name : public EXC {          \
      public:                          \
        name( const char *sInfo =msg ) : \
            EXC( sInfo ) {}              \
        EXC_TYPE( name )               \
    };

/** Define a new exception.
 * Define a new exception class of classname \a name, information \a msg and
 * derivated from exception class parentClass.
 * @param name name of the class.
 * @param msg information for the class.
 * @param parentClass parent classname exception.
 */
#define NEWEXCD(name, msg, parentClass) \
    class name : public parentClass {   \
      public:                           \
        name( const char *sInfo =msg ) :  \
            parentClass( sInfo ) {}       \
        EXC_TYPE( name )                \
    };

// Types:
// #####################Last change: 30/09/97 19h04mn#########################

/*! \Brief Base class for all exception objects.
 */
class EXC {
  public:
    CLASS_NAME( EXC );

    /*! Constructor.
     * @param sInfo information about the exception (name...).
     * @param nLine source line where the exception has been raised.
     * @param sSourceFile source file where the exception has been raised.
     */
    EXC( const char *sInfo =NULL,
         SL nLine =0,
         const char *sSourceFile =NULL);

    /*! Set exception raising context.
     * Set the raising context of the exception.
     * @param nLine source line where the exception is raised.
     * @param sSourceFile source file where the exception is raised.
     * @return a pointer on the exception (so it can be pass to throw).
     */
    EXC &setContext( UL nLine,
                     const char *sSourceFile );

    /*! Return the information about the exception.
     * @return exception information.
     */
    const char *info() { return _sInfo; }

    /*! Return the line number in the source file where the exception was raised.
     * @return line.
     */
    SL line() { return _nLine; }

    /*! Return the source file where the exception was raised.
     * @return source file.
     */
    const char *sSourceFile() { return _sSourceFile; }

    /*! Dump information on stderr.
     */
    virtual void dump();

  private:
    /*! Information about the exception.
     */
    const char *_sInfo;

    /*! Source line where the exception has been raised.
     */
    SL _nLine;

    /*! Source file where the exception has been raised.
     */
    char _sSourceFile[512];
};

// inlines:
// #####################Last change: 22/12/95 18h37mn#########################

inline EXC::EXC( const char *sInfo,
                 SL nLine,
                 const char *sSourceFile )
{
  _sInfo = sInfo;
  setContext( nLine, sSourceFile );
}



#endif
