// TfwkArrayPtrT.h: interface for the TfwkArrayPtrT class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TFWKARRAYPTRT_H__1D7D9218_4132_11D3_BE28_0040053AD648__INCLUDED_)
#define AFX_TFWKARRAYPTRT_H__1D7D9218_4132_11D3_BE28_0040053AD648__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "TfwkArrayT.h"

/*! class TfwkArrayPtrT<Obj>
 * \include "TfwkArrayPtrT.h"
 * \brief This template represents an array of pointer.
 *
 * All the object stored in the array are destroyed with the array.
 */
template<class Obj>
class TfwkArrayPtrT : public TfwkArrayT<Obj>
{
public:
  typedef TfwkArrayT<Obj> SuperClass;

  /*!
   * Constructs the array.
   */
  TfwkArrayPtrT() {}

  /*!
   * Destructs all the pointed objects.
   */
	virtual ~TfwkArrayPtrT();

  /*!
   * Removes an item at a given position.
   * @param nPos [In] Position of the item to remove.
   */
  virtual void remove( int nPos );

  /*!
   * Removes all the item and destroyes them.
   */
  virtual void clear();
};



// Template implementation : TfwkArrayPtrT<Obj>
// -----------------------------------------

template<class Obj>
TfwkArrayPtrT<Obj>::~TfwkArrayPtrT()
{
  clear();
}


template<class Obj>
void 
TfwkArrayPtrT<Obj>::remove( int nPos )
{
  Obj &pObj = at( nPos );
  SuperClass::remove( nPos );
  if ( pObj )
    delete pObj;
}


template<class Obj>
void 
TfwkArrayPtrT<Obj>::clear()
{
  int nCount = length()-1;

  for ( ; nCount >= 0; --nCount )
  {
    Obj &pObj = at( nCount );
    if ( pObj )
      delete pObj;
  }

  resize( 0 );
}


#endif // !defined(AFX_TFWKARRAYPTRT_H__1D7D9218_4132_11D3_BE28_0040053AD648__INCLUDED_)
