// TfwkTestReport.h: interface for the TfwkTestReport class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TFWKTESTREPORT_H__EADAD241_4646_11D3_BE38_0040053AD648__INCLUDED_)
#define AFX_TFWKTESTREPORT_H__EADAD241_4646_11D3_BE38_0040053AD648__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

class TFWK_UI_EXPORT TfwkTestReport  
{
public:
  /*!
   * Error type.
   */
  enum ReportError
  {
    /// The test succeed.    
    reNone =0,
    /// An uncaught exception was thrown.
    reException,
    /// An assertion failed.
    reAssertion,
    /// An internal error occured.
    reInternal,
  };

  /*!
   * Constructs an test report.
   */
  TfwkTestReport();

  /*!
   * Destructor.
   */
  virtual ~TfwkTestReport() {}

  /*!
   * The test was run with success.
   */
  virtual void testSuccess();

  /*!
   * An assertion failed while running the test.
   * @param [In] szDetail Expression that caused the test to failed.
   * @param [In] szSource Filename of the source file in which the assertion was.
   * @param [In] nLine Line number of the assertion.
   */
  virtual void testAssert( const char *szDetail,
                           const char *szSource,
                           int nLine );

  /*!
   * An exception was trown while running the test and was not caught.
   */
  virtual void testException();

  /*!
   * An internal error prevented the test to be runned.
   *
   * Error could be that the test does not exist.
   */
  virtual void testInternal( const char *szDetail =NULL,
                             const char *szSource =NULL,
                             int nLine =0 );

  /*!
   * Error type.
   */
  ReportError error() const;

  /*!
   * Filename where the error occured.
   */
  const char *filename() const;

  /*!
   * Detail about the error.
   */
  const char *detail() const;

  /*!
   * Line number where the error occured.
   */
  int line() const;

private:
  /*!
   * Tests status.
   */
  ReportError _nError;

  /*!
   * Information about the error.
   */
  const char *_szDetail;
  
  /*!
   * Source file where the error occured.
   */
  const char *_szSource;

  /*!
   * Line number of the error.
   */
  int _nLine;
};



// Inline: TfwkTestReport
// ----------------------

inline 
TfwkTestReport::TfwkTestReport() : 
    _nError( reNone ),
    _szDetail( NULL ),
    _szSource( NULL ),
    _nLine( 0 )
{
}


inline void 
TfwkTestReport::testSuccess()
{
  _nError = reNone;
  _szDetail = NULL;
  _szSource = NULL;
  _nLine = 0;
}


inline void 
TfwkTestReport::testAssert( const char *szDetail,
                            const char *szSource,
                            int nLine )
{
  _nError = reAssertion;
  _szDetail = szDetail;
  _szSource = szSource;
  _nLine = nLine;
}


inline void 
TfwkTestReport::testException()
{
  _nError = reException;
  _szDetail = NULL;
  _szSource = NULL;
  _nLine = 0;
}


inline void 
TfwkTestReport::testInternal( const char *szDetail,
                              const char *szSource,
                              int nLine )
{
  _nError = reInternal;
  _szDetail = szDetail;
  _szSource = szSource;
  _nLine = nLine;
}


inline TfwkTestReport::ReportError 
TfwkTestReport::error() const
{
  return _nError;
}


inline const char *
TfwkTestReport::filename() const
{
  return _szSource ? _szSource : "";
}


inline const char *
TfwkTestReport::detail() const
{
  return _szDetail ? _szDetail : "";
}


inline int 
TfwkTestReport::line() const
{
  return _nLine;
}



#endif // !defined(AFX_TFWKTESTREPORT_H__EADAD241_4646_11D3_BE38_0040053AD648__INCLUDED_)
