// TfwkUiTest.cpp: implementation of the TfwkUiTest class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TfwkUi.h"
#include "TfwkUiTest.h"
#include "TfwkUiSuite.h"
#include "TfwkUiTestCfg.h"
#include "TfwkUiCfg.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TfwkUiTest::TfwkUiTest( TfwkUiSuite *pSuite,
                        const CString &strClass,
                        const CString &strMethod,
                        TfwkUiSourceId idSource,
                        TfwkUiTestCfg *pUnitTest ) :
    _pSuite( pSuite ),
    _strClass( strClass ),
    _strMethod( strMethod ),
    _idSource( idSource ),
    _pUnitTest( pUnitTest )
{
  ASSERT( pSuite );
  _pSuite->AddTest( this );
  if ( _pUnitTest )
    _pUnitTest->IncSuiteRef();
}


TfwkUiTest::~TfwkUiTest()
{
  if ( _pUnitTest )
    _pUnitTest->DecSuiteRef();
}


void 
TfwkUiTest::RunTest( TfwkUiCfg &cfg,
                     TfwkUiTestSuiteReport &report )
{
  CString strName;
  FullMethodName( strName );

  TfwkUiTestCfg *pTest = _pUnitTest;
  if ( !pTest )   // Tests does not exist
  {
    ++report._nSkipped;
    return;
  }

  const TfwkUnitTestMethodInfo &info = pTest->info();
  TfwkUnitTest *pTestObject =NULL;
  try
  {
    pTestObject = (TfwkUnitTest *)info.unitTest().clsidTest().createObject();
    ++report._nTotal;
    info.run( pTestObject );
    ++report._nSuccess;
  }
  catch( TfwkExc *pExc )
  {
    if ( pExc->reason() == TfwkExc::trAssertionFailed )
    {
      ++report._nFailure;
      report.ReportError( etpFailure,
                          pExc->source(),
                          pExc->line(),
                          pExc->info(),
                          this );
    }
    else                    // internal error
      ++report._nSkipped;
  }
  catch( ... )
  {
    ++report._nError;       // uncatched exception
    report.ReportError( etpError,
                        _T( "" ),
                        0,
                        _T( "" ),
                        this );
  }

  if ( pTestObject )
    delete pTestObject;
}
