// TtrAutoDetectCtx.h: interface for the TtrAutoDetectCtx class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TTRAUTODETECTCTX_H__937EFEFE_AB0A_4F61_B084_536CBE6316D1__INCLUDED_)
#define AFX_TTRAUTODETECTCTX_H__937EFEFE_AB0A_4F61_B084_536CBE6316D1__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

class TtrTextBlock;
class TtrTextLine;
class TtrFormatSetting;



/*! \class TtrAutoDetectCtx
 * \brief This class represents the context of the autodetection.
 */
class TtrAutoDetectCtx
{
public:
	TtrAutoDetectCtx();

	virtual ~TtrAutoDetectCtx();

  /*! Analyses the given block of text to autodetect the format settings.
   * @param block Block of text to analyse.
   */
  void analyse( const TtrTextBlock &block );

  /*! Guess the format settings.
   *
   * Guess what the format settings should be based on the result of the
   * analysis of the text.
   *
   * @param setting Setting to set.
   */
  void guessSetting( TtrFormatSetting &setting );

private:
  /*! Resets the analyse.
   * Put all the miscellanous analysis statistic to their initial state.
   */
  void resetAnalyse();

  /*! Teste si la ligne commence par une minuscule.
   */
  bool isLowerCaseLine( const TtrTextLine &line );

  /*! Set the "Script" setting.
   * @param setting Setting to set.
   */
  void setupScriptSetting( TtrFormatSetting &setting );

  /*! Initializes the sorted line information collection.
   */
  void makeSortedLineInfo();


private:
  struct IndentLineInfo
  {
    IndentLineInfo( SL nIndentLevel =0,
                    SL nCount =0,
                    SL nCountStartWithLowerCase =0 ) :
        _nIndentLevel( nIndentLevel ),
        _nCount( nCount ),
        _nCountStartWithLowerCase( nCountStartWithLowerCase ) {}

    /*! Indentation level.
     */
    SL _nIndentLevel;

    /*! Number of lines with that indentation level.
     */
    SL _nCount;

    /*! Number of line with that indentation level that starts with a lower case.
     */
    SL _nCountStartWithLowerCase;
  };

  struct FnLessLineInfo : public EFn2T<IndentLineInfo,IndentLineInfo,bool>
  {
    bool operator ()( const IndentLineInfo &arg1,
                      const IndentLineInfo &arg2 ) const
    {
      return arg1._nCount < arg2._nCount;
    }
  };

  struct FnEqualLineInfo : public EFn2T<IndentLineInfo,IndentLineInfo,bool>
  {
    bool operator ()( const IndentLineInfo &arg1,
                      const IndentLineInfo &arg2 ) const
    {
      return arg1._nCount == arg2._nCount;
    }
  };

private:
  typedef EHashT<SL,IndentLineInfo,SL,const IndentLineInfo &> HashIndentOcc;

  /*! Hash-table that contains the number of occurence of each indentation size.
   */
  HashIndentOcc _hashIndentOcc;

  /*! Number of lines that contains ':'
   */
  SL _nCountLineContainColon;

  /*! Number of line that match the 'half blank pattern' (every other line is a 
   *  blank line).
   */
  SL _nCountHalfBlankPattern;

  /*! Number of lines in the analysed text.
   */
  SL _nLineCount;

  typedef EBinDynArrayT<IndentLineInfo> ArrayLineInfo;

  typedef ESortedCollectionT<IndentLineInfo, 
                             ArrayLineInfo, 
                             FnLessLineInfo, 
                             FnEqualLineInfo> SortedLineInfo;

  SortedLineInfo _sortedLineInfo;
};

#endif // !defined(AFX_TTRAUTODETECTCTX_H__937EFEFE_AB0A_4F61_B084_536CBE6316D1__INCLUDED_)
