// TtrDlgTextComposite.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "TtrDlgTextComposite.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// TtrDlgTextComposite dialog


TtrDlgTextComposite::TtrDlgTextComposite(CWnd* pParent ) : 
    CDialog(IDD_TEXT_COMPOSITE, pParent)
{
	//{{AFX_DATA_INIT(TtrDlgTextComposite)
	//}}AFX_DATA_INIT
}


void 
TtrDlgTextComposite::setCoreInfoList( const TtrTextCoreInfoList &listCoreInfo )
{
  _vCoreInfo.duplicate( listCoreInfo );
}


void 
TtrDlgTextComposite::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(TtrDlgTextComposite)
	DDX_Control(pDX, IDC_LIST_TEXT_COMPOSITE, _lvText);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(TtrDlgTextComposite, CDialog)
	//{{AFX_MSG_MAP(TtrDlgTextComposite)
	ON_BN_CLICKED(IDC_TEXT_COMPOSITE_MOVE_UP, OnTextCompositeMoveUp)
	ON_BN_CLICKED(IDC_TEXT_COMPOSITE_MOVE_DOWN, OnTextCompositeMoveDown)
	ON_BN_CLICKED(IDOK, OnOk)
	ON_BN_CLICKED(IDC_TEXT_MAKE_COMPOSITE, OnTextMakeComposite)
	ON_BN_CLICKED(IDC_TEXT_MAKE_SUB_TEXT, OnTextMakeSubText)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// TtrDlgTextComposite message handlers

void 
TtrDlgTextComposite::OnTextCompositeMoveUp() 
{
  TtrTextCoreInfoList listInfo;
  getSelectedTextList( listInfo );
  for ( SL nIdx =0; nIdx < listInfo.count(); ++nIdx )
  {
    SL nPos = findItemPos( listInfo[nIdx] );
    if ( nPos == 0 )
      continue;
    switchItem( nPos-1, nPos );
  }
  refreshList();
}


void 
TtrDlgTextComposite::OnTextCompositeMoveDown() 
{
  TtrTextCoreInfoList listInfo;
  getSelectedTextList( listInfo );
  for ( SL nIdx =listInfo.count()-1; nIdx >= 0; --nIdx )
  {
    SL nPos = findItemPos( listInfo[nIdx] );
    if ( nPos >= _lvText.GetItemCount()-1 )
      continue;
    switchItem( nPos, nPos+1 );
  }
  refreshList();
}


void 
TtrDlgTextComposite::OnOk() 
{
  CDialog::OnOK();
}


BOOL 
TtrDlgTextComposite::OnInitDialog() 
{
	CDialog::OnInitDialog();

  setupListFormat();
  refreshList();
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}


void 
TtrDlgTextComposite::refreshItem( TtrTextCoreInfo *pInfo )
{
  if ( !_lvText.GetSafeHwnd() )
    return;

  SL nItemCount = _lvText.GetItemCount();
  for ( SL nIdx =0; nIdx < nItemCount; ++nIdx )
  {
    if ( (TtrTextCoreInfo *)_lvText.GetItemData( nIdx ) == pInfo )
    {
      setListItem( nIdx, *pInfo, false );
      break;
    }
  }
}



void 
TtrDlgTextComposite::setupListFormat()
{
  _imlText.Create( IDB_TEXT_COMPOSITE, 16, 1, RGB( 255,0,255 ) );

  CListCtrl &list = _lvText;

  list.SetImageList( &_imlText, LVSIL_NORMAL );
  list.SetImageList( &_imlText, LVSIL_SMALL );

  LV_COLUMN lvCol;
  lvCol.mask = LVCF_FMT | LVCF_SUBITEM | LVCF_TEXT | LVCF_WIDTH;
  lvCol.fmt = LVCFMT_LEFT;

  CString str;
  char *s;

  VERIFY( str.LoadString( IDS_FILENAME ) );
  s = str.GetBuffer( str.GetLength() );
  lvCol.pszText = s ; 
	lvCol.iSubItem = 0 ;
  lvCol.cx = 800;
	list.InsertColumn( 0, &lvCol ) ;
  str.ReleaseBuffer();
}


void 
TtrDlgTextComposite::refreshList()
{
  CListCtrl &list = _lvText;
  list.DeleteAllItems();

  for ( SL nIdx =0; nIdx < _vCoreInfo.count(); ++nIdx )
  {
    setListItem( list.GetItemCount(), *_vCoreInfo[ nIdx ] );
  }
}


void 
TtrDlgTextComposite::setListItem( SL nLine, 
                                  TtrTextCoreInfo &info, 
                                  bool bIsNewItem )
{
  CListCtrl &list = _lvText;
  if ( !list.GetSafeHwnd() )
    return;

  EString strText;
  if ( info.getCompositeParentId() != TtrTextCoreInfo::nullId )
    strText += "    >>";

  EFilename filename( info.getName() );
  strText += filename.name();

  char *s;
  CString str;
  LV_ITEM		lvItem ;
  str = (LPCTSTR)strText;
  s = str.GetBuffer( str.GetLength() );
  lvItem.mask = LVIF_TEXT | LVIF_IMAGE;
  lvItem.iItem = nLine; lvItem.iSubItem = 0;
  lvItem.state = 0; lvItem.stateMask = 0;
  lvItem.pszText = s;
  lvItem.lParam = NULL;
  lvItem.iImage = ( info.getCompositeParentId() != TtrTextCoreInfo::nullId ) ?
                  imgSubText : imgComposite;
  if ( bIsNewItem )    
    nLine = list.InsertItem( &lvItem );
  else
    nLine = list.SetItem( &lvItem );
  list.SetItemData( nLine, (UL)&info );
  str.ReleaseBuffer();
}




void 
TtrDlgTextComposite::getSelectedTextList( TtrTextCoreInfoList &listInfo )
{
  int nStart =-1;
  while ( (nStart = _lvText.GetNextItem( nStart, LVNI_SELECTED )) >= 0 )
  {
    listInfo.append( (TtrTextCoreInfo *)_lvText.GetItemData( nStart ) );
  }
}


TtrTextCoreInfoId 
TtrDlgTextComposite::findParentCompositeFor( TtrTextCoreInfo *pForInfo )
{
  SL nIdxInfo = findItemPos( pForInfo );

  SL nItemCount = _lvText.GetItemCount();
  for  ( SL nIdx =nIdxInfo-1; nIdx >=0; --nIdx )
  {
    TtrTextCoreInfo *pInfo = (TtrTextCoreInfo *)_lvText.GetItemData( nIdx );
    if ( pInfo->getCompositeParentId() == TtrTextCoreInfo::nullId )
      return pInfo->getId();
  }
  
  return TtrTextCoreInfo::nullId;
}


void 
TtrDlgTextComposite::OnTextMakeComposite() 
{
  TtrTextCoreInfoList listInfo;
  getSelectedTextList( listInfo );
  for ( SL nIdx =0; nIdx < listInfo.count(); ++nIdx )
  {
    listInfo[nIdx]->setCompositeParentId( TtrTextCoreInfo::nullId );
    refreshItem( listInfo[nIdx] );
  }
}


void 
TtrDlgTextComposite::OnTextMakeSubText() 
{
  TtrTextCoreInfoList listInfo;
  getSelectedTextList( listInfo );
  for ( SL nIdx =0; nIdx < listInfo.count(); ++nIdx )
  {
    if ( listInfo[nIdx] != _vCoreInfo[0] )
      listInfo[nIdx]->setCompositeParentId( findParentCompositeFor( listInfo[nIdx] ) );
    else
      listInfo[nIdx]->setCompositeParentId( TtrTextCoreInfo::nullId );
    refreshItem( listInfo[nIdx] );
  }
}


SL 
TtrDlgTextComposite::findItemPos( TtrTextCoreInfo *pInfo )
{
  LV_FINDINFO infoFind;
  infoFind.flags = LVFI_PARAM;
  infoFind.lParam = (LPARAM)pInfo;
  return _lvText.FindItem( &infoFind, -1 );
}


TtrTextCoreInfo *
TtrDlgTextComposite::getItemAt( SL nIdx )
{
  return (TtrTextCoreInfo *)_lvText.GetItemData( nIdx );
}


void
TtrDlgTextComposite::switchItem( SL nIdx1, SL nIdx2 )
{
  TtrTextCoreInfo *pInfo1 = getItemAt( nIdx1 );
  TtrTextCoreInfo *pInfo2 = getItemAt( nIdx2 );
  setListItem( nIdx1, *pInfo2, false );
  setListItem( nIdx2, *pInfo1, false );
  _vCoreInfo[ nIdx1 ] = pInfo2;
  _vCoreInfo[ nIdx2 ] = pInfo1;
}
