// //////////////////////////////////////////////////////////////////////////
// Implementation file TtrFileGroupMaker.cpp for class TtrFileGroupMaker
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2000/09/09
// //////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "TtrFileGroupMaker.h"


TtrFileGroupMaker::TtrFileGroupMaker()
{
}


TtrFileGroupMaker::~TtrFileGroupMaker()
{
}


void 
TtrFileGroupMaker::setupFromDir( const EString strDirName )
{
  EDirectory dir( strDirName );
  dir.findFile( _aFile );
  findGroup();
}


void 
TtrFileGroupMaker::findGroup()
{
  removeDirectories();

  for ( SL nIdx =0; nIdx < _aFile.count(); )
  {
    const EFilename &filename = _aFile[ nIdx ];
    EString strRadix = extractRadix( filename );
    SL nFileCount = findRadixCount( strRadix );
    if ( nFileCount > 1 )
    {
      groupFileWithRadix( strRadix, nIdx );
//      TRACE( "%d files found, remaining %d, idx = %d\n", 
//             nFileCount, _aFile.count(), nIdx );
    }
    else 
      ++nIdx;
  }

  while ( _aFile.count() > 0 )
  {
    SL nIdx = 0;
    groupFileWithRadix( _aFile.first().basename(), nIdx );
  }
}


EString 
TtrFileGroupMaker::extractRadix( const EFilename &filename ) const
{
  static EString strNum( "012345679" );
  EString strName = filename.basename();
  SL nIdx = strName.length();
  while ( nIdx > 0  &&
          strNum.pos( strName.at( --nIdx ) ) >= 0 );

  if ( nIdx < 0 )
    return strName;
  return strName.left( nIdx +1 );
}


SL 
TtrFileGroupMaker::findRadixCount( const EString &strRadix ) const
{
  SL nCount =0;
  for ( SL nIdx =0; nIdx < _aFile.count(); ++nIdx )
    if ( _aFile[ nIdx ].basename().left( strRadix.length() ) == strRadix )
      ++nCount;
  return nCount;
}


void 
TtrFileGroupMaker::groupFileWithRadix( const EString &strRadix,
                                       SL &nIdx )
{
  Info *pInfo = new Info();
  pInfo->_strRadix = strRadix;
  _apInfo.append( pInfo );

  for ( SL nIdxFile =0; nIdxFile < _aFile.count(); )
  {
    const EFilename &filename = _aFile[ nIdxFile ];
    if ( filename.basename().left( strRadix.length() ) == strRadix )
    {
      pInfo->_aFilename.append( filename );
      _aFile.remove( nIdxFile );
      if ( nIdxFile < nIdx )
        --nIdx;
    }
    else
      ++nIdxFile;
  }
}


SL 
TtrFileGroupMaker::groupCount() const
{
  return _apInfo.count();
}


const EString &
TtrFileGroupMaker::groupRadix( SL nIdx ) const
{
  return _apInfo[ nIdx ]->_strRadix;
}


const EFilenameList &
TtrFileGroupMaker::groupFilenameList( SL nIdx ) const
{
  return _apInfo[ nIdx ]->_aFilename;
}


void 
TtrFileGroupMaker::removeDirectories()
{
  for ( SL nIdx =0; nIdx < _aFile.count(); )
  {
    if ( _aFile[ nIdx ].isDirectory() )
      _aFile.remove( nIdx );
    else
      ++nIdx;
  }
}
