// //////////////////////////////////////////////////////////////////////////
// Header file TtrFileGroupMaker.h for class TtrFileGroupMaker
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2000/09/09
// //////////////////////////////////////////////////////////////////////////
#ifndef TTRFILEGROUPMAKER_H
#define TTRFILEGROUPMAKER_H


/*! \class TtrFileGroupMaker
 * \brief This class extacts the file that are "filename" connected (same radix...).
 */
class TtrFileGroupMaker
{
public:
  /*! Constructs a TtrFileGroupMaker object.
   */
  TtrFileGroupMaker();

  /*! Destructor.
   */
  virtual ~TtrFileGroupMaker();

  /*! Setup from directory.
   * @param strDirName Name of the directory to examine.
   */
  void setupFromDir( const EString strDirName );

  /*! Computes the groups.
   */
  void findGroup();

  /*! Gets the number of group.
   * @return Number of group.
   */
  SL groupCount() const;

  /*! Gets the radix of the specified group.
   * @param nIdx Index of the group (0 based).
   * @return Radix of the specified group.
   */
  const EString &groupRadix( SL nIdx ) const;

  /*! Gets the list of filename of the specified group.
   * @param nIdx Index of the group (0 based).
   * @return Reference on the list of filename.
   */
  const EFilenameList &groupFilenameList( SL nIdx ) const;

private:
  /*! Prevents the use of the copy constructor.
   */
  TtrFileGroupMaker( const TtrFileGroupMaker &copy );

  /*! Prevents the use of the copy operator.
   */
  TtrFileGroupMaker &operator =( const TtrFileGroupMaker &copy );

private:
  /*! Extract the radix of the filename.
   */
  EString extractRadix( const EFilename &filename ) const;

  /*! Find the number of file with the specified radix.
   */
  SL findRadixCount( const EString &strRadix ) const;

  /*! Transfert the file with the specified radix into a new group.
   * @param strRadix Radix of the file to extract.
   * @param nIdx Index maintains to the file currently pointed.
   */
  void groupFileWithRadix( const EString &strRadix,
                           SL &nIdx );

  /*! Supprime les rpertoires de la liste.
   */
  void removeDirectories();

private:
  struct Info
  {
    EString _strRadix;
    EFilenameList _aFilename;
  };

private:
  /*! Array of files to group.
   */
  EFilenameList _aFile;

  typedef EPtrDynArrayT<Info *> ArrayInfo;
  /*! Array of filelist.
   */
  ArrayInfo _apInfo;


};



// Inlines methods for TtrFileGroupMaker:
// --------------------------------------



#endif  // TTRFILEGROUPMAKER_H
