// //////////////////////////////////////////////////////////////////////////
// Header file TtrFmtElt.h for class TtrFmtElt
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2000/09/15
// //////////////////////////////////////////////////////////////////////////
#ifndef TTRFMTELT_H
#define TTRFMTELT_H

#include "TtrFmtEltVisitor.h"
class TtrFmtElt;

/*! Auto-pointer on formatted text elements.
 */
typedef EAutoPtrT<TtrFmtElt> TtrFmtEltPtr;


/*! \class TtrFmtElt
 * \brief This class represents a element of  a formatted text.
 */
class TtrFmtElt
{
public:
  enum EltType
  {
    eltBlank,
    eltPara,
  };


  /*! Constructs a TtrFmtElt object.
   */
  TtrFmtElt();

  /*! Destructor.
   */
  virtual ~TtrFmtElt();

  /*! Gets the type of the element.
   * @return Type of the element.
   */
  virtual EltType type() const =0;

  /*! Visits the element.
   * @param visitor Visitor that is visited by the element.
   */
  virtual void visit( TtrFmtEltVisitor &visitor ) =0;

private:
  /*! Prevents the use of the copy constructor.
   */
  TtrFmtElt( const TtrFmtElt &copy );

  /*! Prevents the use of the copy operator.
   */
  TtrFmtElt &operator =( const TtrFmtElt &copy );

private:
};



// Inlines methods for TtrFmtElt:
// ------------------------------



#endif  // TTRFMTELT_H
