// //////////////////////////////////////////////////////////////////////////
// Header file TtrFmtIndentInfo.h for class TtrFmtIndentInfo
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2000/09/16
// //////////////////////////////////////////////////////////////////////////
#ifndef TTRFMTINDENTINFO_H
#define TTRFMTINDENTINFO_H


/*! \class TtrFmtIndentInfo
 * \brief This class represents the information about the indent of an element.
 */
class TtrFmtIndentInfo
{
public:
  /*! Constructs a TtrFmtIndentInfo object.
   */
  TtrFmtIndentInfo();

  /*! Copy constructor.
   * @param copy Object to copy.
   */
  TtrFmtIndentInfo( const TtrFmtIndentInfo &copy );

  /*! Destructor.
   */
  virtual ~TtrFmtIndentInfo();

  /*! Copy operator.
   * @param copy Object to copy.
   * @return Reference on this object.
   */
  TtrFmtIndentInfo &operator =( const TtrFmtIndentInfo &copy );

  /*! Sets the indentation level.
   * @param nIndentLevel 0 indicates that the line is not "indented" (keep its
   *                     original indentation).
   *                     >0 indicates that the line should be indented.
   */
  void setIndentLevel( SL nIndentLevel );

  /*! Sets the indent string.
   * @param strIndent Indent string, the string that is at the beginning of the
   *                  line and match the indentation characters.
   */
  void setIndentString( const EString &strIndent );

  /*! Returns the indent level.
   * @return Indent level.
   */
  SL indentLevel() const;

  /*! Returns the indent string.
   * @return Indent string.
   */
  const EString &indentString() const;

private:
  /*! Indent level.
   */
  SL _nIndentLevel;

  /*! Indent string.
   */
  EString _strIndent;
};



// Inlines methods for TtrFmtIndentInfo:
// -------------------------------------


inline void 
TtrFmtIndentInfo::setIndentLevel( SL nIndentLevel )
{
  _nIndentLevel = nIndentLevel;
}


inline void 
TtrFmtIndentInfo::setIndentString( const EString &strIndent )
{
  _strIndent = strIndent;
}


inline SL 
TtrFmtIndentInfo::indentLevel() const
{
  return _nIndentLevel;
}


inline const EString &
TtrFmtIndentInfo::indentString() const
{
  return _strIndent;
}



#endif  // TTRFMTINDENTINFO_H
