// //////////////////////////////////////////////////////////////////////////
// Header file TtrFmtPara.h for class TtrFmtPara
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2000/09/14
// //////////////////////////////////////////////////////////////////////////
#ifndef TTRFMTPARA_H
#define TTRFMTPARA_H

#include "TtrFmtElt.h"
#include "TtrFmtIndentInfo.h"
#include "TtrTagList.h"


/*! \class TtrFmtPara
 * \brief This class represents a paragraph of a formatted text.
 */
class TtrFmtPara : public TtrFmtElt
{
public:
  /*! Constructs a TtrFmtPara object.
   */
  TtrFmtPara();

  /*! Destructor.
   */
  virtual ~TtrFmtPara();

  /*! Gets the type of the element.
   * @return Type of the element.
   */
  virtual EltType type() const;

  /*! Visits the element.
   * @param visitor Visitor that is visited by the element.
   */
  virtual void visit( TtrFmtEltVisitor &visitor );

  /*! Sets the indent information.
   * @param infoIndent Indent information.
   */
  void setIndent( const TtrFmtIndentInfo &infoIndent );

  /*! Returns the indent information.
   * @return Indent information.
   */
  const TtrFmtIndentInfo &indent() const;

  /*! Sets the paragraph content.
   * @param strContent Paragraph content.
   */
  void setContent( const EString &strContent );

  /*! Returns the content of the paragraph.
   * @returns Content of the paragraph.
   */
  const EString &content() const;

  /*! Returns the tag list associated to the paragraph.
   * @return Tag list associated to the paragraph.
   */
  TtrTagList &tagList();

  /*! Returns the tag list associated to the paragraph.
   * @return Tag list associated to the paragraph.
   */
  const TtrTagList &tagList() const;

private:
  /*! Prevents the use of the copy constructor.
   */
  TtrFmtPara( const TtrFmtPara &copy );

  /*! Prevents the use of the copy operator.
   */
  TtrFmtPara &operator =( const TtrFmtPara &copy );

private:
  TtrFmtIndentInfo _infoIndent;
  EString _strContent;

  /*! List of tags associated to this paragraph.
   */
  TtrTagList _listTag;
};



// Inlines methods for TtrFmtPara:
// -------------------------------


inline const EString &
TtrFmtPara::content() const
{
  return _strContent;
}


inline void 
TtrFmtPara::setIndent( const TtrFmtIndentInfo &infoIndent )
{
  _infoIndent = infoIndent;
}


inline void 
TtrFmtPara::setContent( const EString &strContent )
{
  _strContent = strContent;
}


inline const TtrFmtIndentInfo &
TtrFmtPara::indent() const
{
  return _infoIndent;
}


inline TtrTagList &
TtrFmtPara::tagList()
{
  return _listTag;
}


inline const TtrTagList &
TtrFmtPara::tagList() const
{
  return _listTag;
}



#endif  // TTRFMTPARA_H
