// //////////////////////////////////////////////////////////////////////////
// Header file TtrFmtTextBuilder.h for class TtrFmtTextBuilder
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2000/09/16
// //////////////////////////////////////////////////////////////////////////
#ifndef TTRFMTTEXTBUILDER_H
#define TTRFMTTEXTBUILDER_H


#include "TtrFmtElt.h"
class TtrFmtText;
class TtrTextLine;
class TtrWriterSetting;


/*! \class TtrFmtTextBuilder
 * \brief This class is a builder to constructs formatted text.
 */
class TtrFmtTextBuilder
{
public:
  /*! Constructs a TtrFmtTextBuilder object.
   * @param text Text that is built.
   * @param setting Writing setting.
   */
  TtrFmtTextBuilder( TtrFmtText &text,
                     const TtrWriterSetting &setting );

  /*! Destructor.
   */
  virtual ~TtrFmtTextBuilder();

  void buildBlank( const TtrTextLine &line );

  void buildPara( const TtrTextLine &line );

  void setIndent( const TtrTextLine &line );

  void addLineToElt( const TtrTextLine &line );

  TtrFmtElt &getElt();

  void addElt();

  void dismissElt();

private:
  /*! Prevents the use of the copy constructor.
   */
  TtrFmtTextBuilder( const TtrFmtTextBuilder &copy );

  /*! Prevents the use of the copy operator.
   */
  TtrFmtTextBuilder &operator =( const TtrFmtTextBuilder &copy );

private:
  /*! Visitor to set the indentation of the element.
   * Should be a local class but VC++ 5.0 go crazy...
   */
  class VisitorIndent;

  /*! Visitor to add a line to the element.
   * Should be a local class but VC++ 5.0 go crazy...
   */
  class VisitorAddLine;

private:
   TtrFmtText *_pText; 

   TtrFmtEltPtr _ptrElt;

   /*! Indicates if line ending with an hyphen ( '-' ) must be concatened
    * with the following line once the hyphen has been removed.
    */
   bool _bRemoveEndingHyphen;

   /*! Indicates if a blank line must be added after each paragraph.
    */
   bool _bJumpOneLine;
};



// Inlines methods for TtrFmtTextBuilder:
// --------------------------------------



#endif  // TTRFMTTEXTBUILDER_H
