// TtrGlobalSetting.cpp: implementation of the TtrGlobalSetting class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TtrGlobalSetting.h"
#include "TtrGlobalSettingListener.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TtrGlobalSetting::TtrGlobalSetting()
{
  SetToDefault();
}


TtrGlobalSetting::~TtrGlobalSetting()
{

}


void 
TtrGlobalSetting::SetToDefault()
{
  setDefaultWriterSetting();
  setDefaultFormatSetting();

  // Others
  m_lsWordEOS.RemoveAll();
}


void 
TtrGlobalSetting::reset()
{
  setDefaultWriterSetting();
  setFormatIndentBlankLine( false );
  setFormatIndentIndentedLine( false );
  setFormatIndentPreviousLineIsBlank( false );
  setFormatIndentSeparator( false );
  setFormatSeparatorChar( _T( "" ) );
  setFormatIndentContainDoubleDot( false );
  setFormatIndentBeginWith( false );
  setFormatIndentBeginWithChar( _T( "" ) );
  setFormatModeEosChar( _T("") );
  setFormatModeEos( false );
  setFormatUseEnclosure( false );
  setFormatUseLeftMargin( false );
  setFormatLeftMargin( 1 );
  setFormatUseRightMargin( false );
  setFormatRightMargin( 76 );
  getFormatEnclosureList().clear();
  setFormatNotLowerCaseLine( false );
}


TtrGlobalSetting& 
TtrGlobalSetting::operator =(const TtrGlobalSetting & mode)
{
  if ( &mode == this )
    return *this;

  copyWriterSetting( mode );
  copyFormatSetting( mode );
  
  // EOS parameter
  m_lsWordEOS.Copy( mode.m_lsWordEOS );

  return *this;
}

// Check if a EOS word is valid!
bool TtrGlobalSetting::WordEOSValid(const CString & sWordEOS, 
                                const CString& sEOSChars)
{
  CString sBadChar = sEOSChars;
  sBadChar += _T( " \t\n\r" );
  return sWordEOS.FindOneOf( sBadChar ) < 0;   // No bad char found !
}


void 
TtrGlobalSetting::addChangeListener( TtrGlobalSettingListener *pListener )
{
  _listChangeListener.addListener( pListener );
}


void 
TtrGlobalSetting::removeChangeListener( TtrGlobalSettingListener *pListener )
{
  _listChangeListener.removeListener( pListener );
}


void 
TtrGlobalSetting::fireChanged()
{
  ChangeListenerList::ItListener it( _listChangeListener.iterator() );
  for ( ; !it.isEnd(); it.next() )
    it.item()->globalSettingChanged( *this );
}


// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
// TtrWriterSetting Interface
// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void 
TtrGlobalSetting::setWriterType( TtrWriterSettingType nWriterType )
{
  _nWriterType = nWriterType;
}


TtrWriterSettingType 
TtrGlobalSetting::getWriterType() const
{
  return _nWriterType;
}


void 
TtrGlobalSetting::setWriterLineMode( TtrWriterLineMode nWriterLineMode )
{
  _nWriterLineMode = nWriterLineMode;
}


TtrWriterLineMode 
TtrGlobalSetting::getWriterLineMode() const
{
  return _nWriterLineMode;
}


void 
TtrGlobalSetting::setWriterReindentSize( UL nReindentSize )
{
  _nWriterReindentSize = nReindentSize;
}


UL 
TtrGlobalSetting::getWriterReindentSize() const
{
  return _nWriterReindentSize;
}


void 
TtrGlobalSetting::setWriterJumpOneLine( bool bJumpOneLine )
{
  _bWriterJumpOneLine = bJumpOneLine;
}


bool 
TtrGlobalSetting::getWriterJumpOneLine() const
{
  return _bWriterJumpOneLine;
}


void 
TtrGlobalSetting::setWriterRemoveEndingHyphen( bool bRemoveEndingHyphen )
{
  _bWriterRemoveEndingHyphen = bRemoveEndingHyphen;
}


bool 
TtrGlobalSetting::getWriterRemoveEndingHyphen() const
{
  return _bWriterRemoveEndingHyphen;
}




// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
// TtrFormatSetting Interface
// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void 
TtrGlobalSetting::setFormatType( TtrFormatSettingType nType )
{
  _nFormatType = nType;
}


TtrFormatSettingType 
TtrGlobalSetting::getFormatType() const
{
  return _nFormatType;
}


void 
TtrGlobalSetting::setFormatSeparatorChar( const EString &strSeparatorChar )
{
  _strFormatSeparatorChar = strSeparatorChar;
}


const EString &
TtrGlobalSetting::getFormatSeparatorChar() const
{
  return _strFormatSeparatorChar;
}


void 
TtrGlobalSetting::setFormatIndentBlankLine( bool bIndentBlankLine )
{
  _bFormatIndentBlankLine = bIndentBlankLine;
}


bool 
TtrGlobalSetting::getFormatIndentBlankLine() const
{
  return _bFormatIndentBlankLine;
}


void 
TtrGlobalSetting::setFormatModeEos( bool bModeEos )
{
  _bFormatModeEos = bModeEos;
}


bool 
TtrGlobalSetting::getFormatModeEos() const
{
  return _bFormatModeEos;
}


void 
TtrGlobalSetting::setFormatIndentIndentedLine( bool bIndentIndentedLine )
{
  _bFormatIndentIndentedLine = bIndentIndentedLine;
}


bool 
TtrGlobalSetting::getFormatIndentIndentedLine() const
{
  return _bFormatIndentIndentedLine;
}


void 
TtrGlobalSetting::setFormatIndentPreviousLineIsBlank( bool bIndentPreviousLineIsBlank )
{
  _bFormatIndentPreviousLineIsBlank = bIndentPreviousLineIsBlank;
}


bool 
TtrGlobalSetting::getFormatIndentPreviousLineIsBlank() const
{
  return _bFormatIndentPreviousLineIsBlank;
}


void 
TtrGlobalSetting::setFormatIndentSeparator( bool bIndentSeparator )
{
  _bFormatIndentSeparator = bIndentSeparator;
}


bool 
TtrGlobalSetting::getFormatIndentSeparator() const
{
  return _bFormatIndentSeparator;
}


void 
TtrGlobalSetting::setFormatIndentContainDoubleDot( bool bIndentContainDoubleDot )
{
  _bFormatIndentContainDoubleDot = bIndentContainDoubleDot;
}


bool 
TtrGlobalSetting::getFormatIndentContainDoubleDot() const
{
  return _bFormatIndentContainDoubleDot;
}


void 
TtrGlobalSetting::setFormatIndentBeginWith( bool bIndentBeginWith )
{
  _bFormatIndentBeginWith = bIndentBeginWith;
}


bool 
TtrGlobalSetting::getFormatIndentBeginWith() const
{
  return _bFormatIndentBeginWith;
}


void 
TtrGlobalSetting::setFormatIndentBeginWithChar( const EString &strBeginWith )
{
  _strFormatBeginWith = strBeginWith;
}


const EString &
TtrGlobalSetting::getFormatIndentBeginWithChar() const
{
  return _strFormatBeginWith;
}


void 
TtrGlobalSetting::setFormatUseEnclosure( bool bUseEnclosure )
{
  _bFormatUseEnclosure = bUseEnclosure;
}


bool 
TtrGlobalSetting::getFormatUseEnclosure() const
{
  return _bFormatUseEnclosure;
}


void 
TtrGlobalSetting::setFormatEnclosureList( const TtrEnclosureList &listEnclosure )
{
  _listFormatEnclosure = listEnclosure;
}


TtrEnclosureList &
TtrGlobalSetting::getFormatEnclosureList()
{
  return _listFormatEnclosure;
}


const TtrEnclosureList &
TtrGlobalSetting::getFormatEnclosureList() const
{
  return _listFormatEnclosure;
}


void 
TtrGlobalSetting::setFormatUseLeftMargin( bool bUseLeftMargin )
{
  _bFormatUseLeftMargin = bUseLeftMargin;
}


bool 
TtrGlobalSetting::getFormatUseLeftMargin() const
{
  return _bFormatUseLeftMargin;
}


void 
TtrGlobalSetting::setFormatLeftMargin( SL nLeftMargin )
{
  _nFormatLeftMargin = nLeftMargin;
}


SL 
TtrGlobalSetting::getFormatLeftMargin() const
{
  return _nFormatLeftMargin;
}


void 
TtrGlobalSetting::setFormatModeEosChar( const EString &strModeEosChar )
{
  _strFormatModeEosChar = strModeEosChar;
}


const EString &
TtrGlobalSetting::getFormatModeEosChar() const
{
  return _strFormatModeEosChar;
}


void 
TtrGlobalSetting::setFormatUseRightMargin( bool bUseRightMargin )
{
  _bFormatUseRightMargin = bUseRightMargin;
}


bool 
TtrGlobalSetting::getFormatUseRightMargin() const
{
  return _bFormatUseRightMargin;
}


void 
TtrGlobalSetting::setFormatRightMargin( SL nRightMargin )
{
  _nFormatRightMargin = nRightMargin;
}


SL 
TtrGlobalSetting::getFormatRightMargin() const
{
  return _nFormatRightMargin;
}


void 
TtrGlobalSetting::setFormatNotLowerCaseLine( bool bNotLowerCaseLine )
{
  _bFormatNotLowerCaseLine = bNotLowerCaseLine;
}


bool 
TtrGlobalSetting::getFormatNotLowerCaseLine() const
{
  return _bFormatNotLowerCaseLine;
}


void 
TtrGlobalSetting::setNeedFilterIndentString( bool bNeedFilterIndentString )
{
  _bFormatNeedFilterIndentString = bNeedFilterIndentString;
}


bool 
TtrGlobalSetting::getNeedFilterIndentString() const
{
  return _bFormatNeedFilterIndentString;
}
