// //////////////////////////////////////////////////////////////////////////
// Implementation file TtrReformatterFsm.cpp for class TtrReformatterFsm
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2000/09/17
// //////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "TtrReformatterFsm.h"
#include "TtrReformatterState.h"
#include "TtrTextWriter.h"
#include "TtrTextWriterBuilder.h"
#include "TtrWriterSetting.h"
#include "TtrFormatSetting.h"


// ////////////////////////////////////////////////////////////////////
// TtrReformatterFsm
// ////////////////////////////////////////////////////////////////////

TtrReformatterFsm::TtrReformatterFsm() :
    _pState( &TtrReformatterStateInitial::instance() ),
    _pBuilder( NULL ),
    _pText( NULL )
{
}


TtrReformatterFsm::~TtrReformatterFsm()
{
}


void 
TtrReformatterFsm::lineStartBlank()
{
  _pState->startBlank( *this );
}


void 
TtrReformatterFsm::lineStartPara()
{
  _pState->startPara( *this );
}


void 
TtrReformatterFsm::lineContinueBlock()
{
  _pState->continueBlock( *this );
}


void 
TtrReformatterFsm::lineEndOfText()
{
  _pState->endOfText( *this );
}


void 
TtrReformatterFsm::changeState( TtrReformatterState &state )
{
  TtrReformatterState &stateOld = *_pState;
  _pState = &state;
  stateOld.exitState( *this );
  
  _pState->enterState( *this );
}


void 
TtrReformatterFsm::reformat( TtrText &text,           
                             EwgProgressInfo &progress )
{
  EASSERT( _pBuilder );

  progress.setProgressRange( 0, text.nbLine() );
  progress.startProgress();

  changeState( TtrReformatterStateInitial::instance() );

  for ( SL nLineNo =0; nLineNo < text.nbLine(); ++nLineNo )
  {
    const TtrTextLine &line = text.line( nLineNo );
    _pLine = &line;

    if ( ( line.indentLevel() < 0 )  &&  
         ( nLineNo > 0) )
      lineContinueBlock();
    else if ( line.isBlank() )
      lineStartBlank();
    else
      lineStartPara();
  }

  lineEndOfText();

  progress.stopProgress();
}


void 
TtrReformatterFsm::setBuilder( TtrFmtTextBuilder &builder )
{
  _pBuilder = &builder;
}
