// //////////////////////////////////////////////////////////////////////////
// Header file TtrReformatterFsm.h for class TtrReformatterFsm
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2000/09/17
// //////////////////////////////////////////////////////////////////////////
#ifndef TTRREFORMATTERFSM_H
#define TTRREFORMATTERFSM_H


class TtrText;
class TtrTextLine;
class TtrReformatterFsm;
class TtrReformatterState;
class EwgProgressInfo;
class TtrFmtTextBuilder;
class TtrFmtText;


// ////////////////////////////////////////////////////////////////////
// TtrReformatterFsm
// ////////////////////////////////////////////////////////////////////

/*!
 * \brief This class implements a Finite StateMachine to write the text.
 */
class TtrReformatterFsm
{
  friend class TtrReformatterState;
public:
	TtrReformatterFsm();
	virtual ~TtrReformatterFsm();

  /*! Sets the formatted text builder.
   * @builder Reference on the formatted text builder.
   */
  void setBuilder( TtrFmtTextBuilder &builder );

  virtual void reformat( TtrText &text,           
                         EwgProgressInfo &progress );

  /*! Returns the current line.
   * @return The line that is being processed by the FSM.
   */
  const TtrTextLine &line() const;

  /*! Returns the formatted text builder.
   * @return Reference on the formatted text builder.
   */
  TtrFmtTextBuilder &builder() const;

private:
  void changeState( TtrReformatterState &state );

  void lineStartBlank();
  void lineStartPara();
  void lineContinueBlock();
  void lineEndOfText();

private:
  /** @supplierCardinality 1*/
  TtrReformatterState *_pState;
  
  /*! Pointer on the current line.
   */
  const TtrTextLine *_pLine;

  /*! Pointer on the formatted text builder.
   * @supplierCardinality 1
   */
  TtrFmtTextBuilder *_pBuilder;

  /*! Pointer on the formatted text that is created.
   */
  TtrFmtText *_pText;
};




// implemention of TtrReformatterFsm 
// ////////////////////////////////////////////////////////////////////


inline const TtrTextLine &
TtrReformatterFsm::line() const
{
  EASSERT( _pLine );
  return *_pLine;
}


inline TtrFmtTextBuilder &
TtrReformatterFsm::builder() const
{
  EASSERT( _pBuilder );
  return *_pBuilder;
}


#endif  // TTRREFORMATTERFSM_H
