// //////////////////////////////////////////////////////////////////////////
// Implementation file TtrReformatterState.cpp for class TtrReformatterState
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2000/09/17
// //////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "TtrReformatterState.h"
#include "TtrFmtTextBuilder.h"


#undef TTRSTATE
#define TTRSTATE( state ) \
    changeState( fsm,TtrReformatterState##state::instance() )



// ////////////////////////////////////////////////////////////////////
// TtrReformatterStateInitial
// ////////////////////////////////////////////////////////////////////

TtrReformatterStateInitial &
TtrReformatterStateInitial::instance()
{
  static TtrReformatterStateInitial state;
  return state;
}


void 
TtrReformatterStateInitial::startBlank( TtrReformatterFsm &fsm ) 
{
  TTRSTATE( Blank );
}


void 
TtrReformatterStateInitial::startPara( TtrReformatterFsm &fsm ) 
{
  TTRSTATE( Para );
}


void 
TtrReformatterStateInitial::continueBlock( TtrReformatterFsm &fsm )
{
  ASSERT( false );    // should never get there...
  TTRSTATE( Para );
}


void 
TtrReformatterStateInitial::endOfText( TtrReformatterFsm &fsm )
{
}


// ////////////////////////////////////////////////////////////////////
// TtrReformatterStateBlank
// ////////////////////////////////////////////////////////////////////



TtrReformatterStateBlank &
TtrReformatterStateBlank::instance()
{
  static TtrReformatterStateBlank state;
  return state;
}


void 
TtrReformatterStateBlank::enterState( TtrReformatterFsm &fsm )
{
  fsm.builder().buildBlank( fsm.line() );
}


void 
TtrReformatterStateBlank::exitState( TtrReformatterFsm &fsm )
{
  fsm.builder().addElt();
}


void 
TtrReformatterStateBlank::startBlank( TtrReformatterFsm &fsm )
{
  TTRSTATE( Blank );
}


void 
TtrReformatterStateBlank::startPara( TtrReformatterFsm &fsm )
{
  TTRSTATE( Para );
}


void 
TtrReformatterStateBlank::continueBlock( TtrReformatterFsm &fsm )
{
  fsm.builder().addLineToElt( fsm.line() );
}


void 
TtrReformatterStateBlank::endOfText( TtrReformatterFsm &fsm )
{
  fsm.builder().addElt();
  TTRSTATE( Initial );
}




// ////////////////////////////////////////////////////////////////////
// TtrReformatterStatePara
// ////////////////////////////////////////////////////////////////////

TtrReformatterStatePara &
TtrReformatterStatePara::instance()
{
  static TtrReformatterStatePara state;
  return state;
}


void 
TtrReformatterStatePara::enterState( TtrReformatterFsm &fsm )
{
  fsm.builder().buildPara( fsm.line() );
}


void 
TtrReformatterStatePara::exitState( TtrReformatterFsm &fsm )
{
  fsm.builder().addElt();
}


void 
TtrReformatterStatePara::startBlank( TtrReformatterFsm &fsm )
{
  TTRSTATE( Blank );
}


void 
TtrReformatterStatePara::startPara( TtrReformatterFsm &fsm )
{
  TTRSTATE( Para );
}


void 
TtrReformatterStatePara::continueBlock( TtrReformatterFsm &fsm )
{
  fsm.builder().addLineToElt( fsm.line() );
}


void 
TtrReformatterStatePara::endOfText( TtrReformatterFsm &fsm )
{
  fsm.builder().addElt();
  TTRSTATE( Initial );
}
