// TtrTag.h: interface for the TtrTag class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TTRTAG_H__D8CF9383_F53E_436C_AF77_ECDF24203370__INCLUDED_)
#define AFX_TTRTAG_H__D8CF9383_F53E_436C_AF77_ECDF24203370__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "TtrTagAttList.h"


EDECL_REF( TtrTag );


/*! \class TtrTag
 * \brief This class represents a tag.
 */
class TtrTag : public ERefObj
{
public:
  /*! Constructs a tag.
   * @param strTagName Name of the tag.
   * @param bIsEngineDefined Indicates if the tag has been defined by the
   *                         reformatting engine.
   */
	TtrTag( const EString &strTagName,
          const TtrTagAttListRef &refAttList = TtrTagAttListRef::nullObject,
          bool bIsEngineDefined =false );

  /*! Destructor.
   */
	virtual ~TtrTag();

  /*! Gets the tag name;
   * @return Tag name.
   */
  const EString &tagName() const;

  /*! Returns the tag symbol id.
   * @return Tag symbol id.
   */
  ESymbolId sidTagName() const;

  /*! Tests if the tag has been defined by the engine.
   * @return \c true if the tag has been defined by the engine.
   */
  bool isEngineDefined() const;

private:
  /*! Id of the tag name.
   */
  const ESymbolId _sidTagName;

  /*! List of attributs associated to the tag.
   */
  TtrTagAttListRef _refAttList;

  /*! Indicates if the tags has been defined by the engine.
   */
  bool _bIsEngineDefined;
};




inline bool 
TtrTag::isEngineDefined() const
{
  return _bIsEngineDefined;
}


inline ESymbolId 
TtrTag::sidTagName() const
{
  return _sidTagName;
}


#endif // !defined(AFX_TTRTAG_H__D8CF9383_F53E_436C_AF77_ECDF24203370__INCLUDED_)
