// TtrTagList.h: interface for the TtrTagList class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TTRTAGLIST_H__D16D3193_ED2A_4D74_8F86_9CC51D9DDA78__INCLUDED_)
#define AFX_TTRTAGLIST_H__D16D3193_ED2A_4D74_8F86_9CC51D9DDA78__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "TtrTag.h"


/*! \brief This class represents a list of tag.
 */
class TtrTagList
{
public:
  static const TtrTagList nullObject;

	TtrTagList();
	virtual ~TtrTagList();

  /*! Returns the number of tags in the list.
   * @return Number of tags in the list.
   */
  SL tagCount() const;

  /*! Returns a reference on the tag at the specified index.
   * @param nIdx Index of the tag (0 based).
   * @return Reference on the tag of index \a nIdx.
   */
  const TtrTagRef &tagAt( SL nIdx ) const;

  /*! Adds a tag to the list.
   * @param refTag Reference on the tag to add.
   */
  void addTag( const TtrTagRef &refTag );

  /*! Removes a tag from the list.
   * @param refTag Reference on the tag to remove.
   */
  void removeTag( const TtrTagRef &refTag );

  /*! Removes all the tags witht the specified symbol.
   * @param sidTag Symbol id of the tags to remove.
   */
  void removeTag( ESymbolId sidTag );

  /*! Copy a tag list.
   * @param copyFrom Tag list that is copied.
   */
  void copy( const TtrTagList &copyFrom );

  /*! Find a tag.
   * @param sidTag Symbol id of the tag to find.
   * @return Reference on the tag, returns \c TtrTagRef::nullObject if the tag
   *         is not found.
   */
  const TtrTagRef &findTag( ESymbolId sidTag ) const;

private:
  typedef EObjDynArrayT<TtrTagRef> ArrayTag;

  ArrayTag _arefTag;
};



inline SL 
TtrTagList::tagCount() const
{
  return _arefTag.count();
}


inline const TtrTagRef &
TtrTagList::tagAt( SL nIdx ) const
{
  return _arefTag[ nIdx ];
}


#endif // !defined(AFX_TTRTAGLIST_H__D16D3193_ED2A_4D74_8F86_9CC51D9DDA78__INCLUDED_)
