// TtrTagManager.h: interface for the TtrTagManager class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TTRTAGMANAGER_H__EBA30A08_D3DF_4993_8076_9BB654CD40A5__INCLUDED_)
#define AFX_TTRTAGMANAGER_H__EBA30A08_D3DF_4993_8076_9BB654CD40A5__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "TtrTag.h"



/*! \class TtrTagManager
 * \brief This class manages tags.
 *
 * This class implements the LightWeight design pattern to create and share
 * tags.
 *
 */
class TtrTagManager
{
  friend class TtrTag;
  friend class TtrTagAtt;

public:
	TtrTagManager();
	virtual ~TtrTagManager();

  /*! Returns the tag manager.
   * @return Reference on the tag manager.
   */
  static TtrTagManager &manager();

  /*! Retreives a specified tag.
   * @param strTagName Name of the tag.
   * @param refAttList Reference on the list of attribut associated to the tag.
   * @param bIsEngineDefined Indicates if the tag has been defined by the
   *                         reformatting engine.
   * @return Reference pointer on the requested tag.
   */
	TtrTagRef getTag( const EString &strTagName,
                    const TtrTagAttListRef &refAttList = TtrTagAttListRef::nullObject,
                    bool bIsEngineDefined =false );

  /*! Retreives a specified tag.
   * @param sidTag Id of the symbol for the tag name.
   * @param refAttList Reference on the list of attribut associated to the tag.
   * @param bIsEngineDefined Indicates if the tag has been defined by the
   *                         reformatting engine.
   * @return Reference pointer on the requested tag.
   */
	TtrTagRef getTag( const ESymbolId sidTag,
                    const TtrTagAttListRef &refAttList = TtrTagAttListRef::nullObject,
                    bool bIsEngineDefined =false );

  /*! Retreives the id of a given symbol.
   *
   * @param strSymbol Symbol name.
   * @return Id of the symbol.
   */
  ESymbolId symbolId( const EString &strSymbol );

  /*! Get the name of a symbol of a given id.
   * @param idSymbol Id of the symbol.
   * @return Name of the symbol of id \a idSymbol.
   */
  const EString &symbolName( ESymbolId idSymbol );

private:
  

private:
  /* Unique instance of the tag manager.
   */
  static EAutoPtrT<TtrTagManager> _ptrManager;

  /*! Symbol holder.
   */
  ESymbolNcsIdHolder _symbolIdHolder;
};




inline TtrTagManager &
TtrTagManager::manager()
{
  if ( !_ptrManager )
    _ptrManager = EAutoPtrT<TtrTagManager>( new TtrTagManager() );
  return *_ptrManager.get();
}

#endif // !defined(AFX_TTRTAGMANAGER_H__EBA30A08_D3DF_4993_8076_9BB654CD40A5__INCLUDED_)
