// //////////////////////////////////////////////////////////////////////////
// Header file TtrWriterHtml.h for class TtrWriterHtml
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2000/09/17
// //////////////////////////////////////////////////////////////////////////
#ifndef TTRWRITERHTML_H
#define TTRWRITERHTML_H


#include "TtrFmtEltVisitor.h"
class TtrWriterSetting;


/*! \class TtrWriterHtml
 * \brief This class represents a writer that write a formatted text in a HTML file.
 */
class TtrWriterHtml : public TtrFmtEltVisitor
{
public:
  /*! Constructs a TtrWriterHtml object.
   * @param rstreamOut Output text stream.
   * @param setting Writer setting.
   */
  TtrWriterHtml( ETextStream &rstreamOut,
                 const TtrWriterSetting &setting );

  /*! Destructor.
   */
  virtual ~TtrWriterHtml();

  /*! Visits a blank block.
   * @param blank Blank block.
   */
  virtual void visitBlank( TtrFmtBlank &blank );

  /*! Visits a paragraph.
   * @param para Paragraph.
   */
  virtual void visitPara( TtrFmtPara &para );

  /*! Should be called when the writer is started.
   */
  void start();

  /*! Should be called when the writer is done.
   */
  void end();

  /*! Returns the text stream used by the writer.
   * @return Text stream used by the writer.
   */
  ETextStream &stream() const;

private:
  /*! Prevents the use of the copy constructor.
   */
  TtrWriterHtml( const TtrWriterHtml &copy );

  /*! Prevents the use of the copy operator.
   */
  TtrWriterHtml &operator =( const TtrWriterHtml &copy );

  /*! Setups the writer using the writer setting.
   * @param setting Writer setting.
   */
  void setup( const TtrWriterSetting &setting );

  /*! Flush the pending line break if present.
   */
  void flushPendingLineBreak();

  /*! Escapes a string.
   * @param strText String to escape.
   * @return Escaped string.
   */
  static EString escape( const EString &strText );

  /*! Replaces space with non-breakable space in a string.
   * @param strText String to change.
   * @return Modified string.
   */
  static EString makeNbspace( const EString &strText );

  /*! Changes the indent level to the specified level.
   * @param nLevel Level.
   */
  void changeIndentLevelTo( SL nLevel );

private:
  /*! Should we keep the original indentation on indented line ?
   */
  bool _bKeepOriginalIndent;

  /*! String for an indentation level "step", opening tag.
   */
  EString _strIndentStepOpen;

  /*! String for an indentation level "step", closing tag.
   */
  EString _strIndentStepClose;

  /*! Reference on the output text stream.
   */
  ETextStream &_rstreamOut;

  /*! Indicates if an additionnal line break is needed.
   *
   * Sets to \c true at the end of a paragraph.
   */
  bool _bNeedLineBreak;

  /*! Current indent string level.
   */
  SL _nIndentLevel;
};



// Inlines methods for TtrWriterHtml:
// ----------------------------------


inline ETextStream &
TtrWriterHtml::stream() const
{
  return _rstreamOut;
}



#endif  // TTRWRITERHTML_H
