// //////////////////////////////////////////////////////////////////////////
// Implementation file TtrWriterTxt.cpp for class TtrWriterTxt
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2000/09/17
// //////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "TtrWriterTxt.h"
#include "TtrWriterSetting.h"
#include "TtrFmtBlank.h"
#include "TtrFmtPara.h"
#include "TtrTagDef.h"


TtrWriterTxt::TtrWriterTxt( ETextStream &rstreamOut,
                           const TtrWriterSetting &setting ) :
  _rstreamOut( rstreamOut )
{
  setup( setting );
}


TtrWriterTxt::~TtrWriterTxt()
{
}

void 
TtrWriterTxt::start()
{
}


void 
TtrWriterTxt::end()
{
}


void 
TtrWriterTxt::setup( const TtrWriterSetting &setting )
{
  _bKeepOriginalIndent = ( setting.getWriterLineMode() == wslmIndentAsOriginal );
  
  if ( _bKeepOriginalIndent )
    _strIndentStep = "";
  else
  {
    bool bUseTab = ( setting.getWriterLineMode() == wslmIndentWithTab );
    _strIndentStep.setNull();
    _strIndentStep.append( bUseTab ? '\t' : ' ', 
                           setting.getWriterReindentSize() );
  }
}


void 
TtrWriterTxt::visitBlank( TtrFmtBlank &blank )
{
  SL nCount = blank.lineCount();
  while ( --nCount >= 0 )
  {
    stream()  <<  endl;
  }
}


void 
TtrWriterTxt::visitPara( TtrFmtPara &para )
{
  if ( para.tagList().findTag( TtrTagDef::sidTagCenter ).isNull() ) // not centered
  {       // indent the paragraph
    if ( _bKeepOriginalIndent )
      stream()  <<  para.indent().indentString();
    else
    {
      EString strIndent;
      strIndent.append( _strIndentStep, para.indent().indentLevel() );
      stream()  <<  strIndent;
    }
    stream()  <<  para.content()
              <<  endl;
  }
  else    // centered
    stream()  <<  para.content()  <<  endl;
}
